/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include <mustux.h>
#include "Audio.hh"


// FOR LATER ::#include "GlobalSettings.hh"

Audio::Audio(Song* pParentSong)
        {
        PENTERCONS;
        peak=0;
        prafFile=(PrafFile*)0;
        wavFile=(WavFile*)0;
        parentSong = pParentSong;
        PEXITCONS;
        }

Audio::~Audio()
        {
        PENTERDES;
        if (prafFile) delete prafFile;
        if (wavFile) delete wavFile;
        if (peak) delete peak;
        PEXITDES;
        }


int Audio::build(QString pRootDir, QString pFilename)
        {
        PENTER;
        rootDir = pRootDir;
        filename = pFilename;
        if (filename.find("praf",0,false)>=0)
                audioType = PRAF;
        else
        if (filename.find("mp3" ,0,false)>=0)
                audioType = MPEG3;
        else
        if (filename.find("ogg", 0,false)>=0)
                audioType = OGGVORBIS;
        else
        if (filename.find("wav", 0,false)>=0)
                audioType = WAV;

        QString absoluteFilename = rootDir + "/" + filename;

        switch (audioType)
                {
                case PRAF :
                        {
                        PrafFile *pf;
                        pf = new PrafFile();
                        if (pf->open((char*)absoluteFilename.latin1())<0)
                                {
                                PERROR( "%s not Found",(const char*)absoluteFilename.latin1());
                                delete pf;
                                PEXIT;
                                return -1;
                                }
                        file = pf->get_file();
                        std::rewind(file);
                        pf->read_header();
                        headerSize = pf->get_header_size();
                        audioSize = pf->get_audio_size();
                        channels = pf->get_channels();
                        rate = pf->get_rate();
                        bitDepth = pf->get_bit_depth();
                        fileSize = audioSize + headerSize;
                        bytesPerSec = rate * channels * (bitDepth / 8);
                        blockSize = channels * (bitDepth / 8);
                        sampleSize = blockSize / channels;
                        totalBlocks = audioSize / blockSize;
                        show_info();
                        peak = new Peak();
                        peak->build(this);
                        rewind();
                        prafFile=pf;
                        wavFile=0;
                        break;
                        }
                case WAV:
                        {
                        WavFile *wf;
                        wf = new WavFile();
                        if (wf->open((char*)absoluteFilename.latin1())<0)
                                {
                                PERROR( "%s not Found",(const char*)absoluteFilename.latin1());
                                delete wf;
                                PEXIT;
                                return -1;
                                }
                        file = wf->get_file();
                        std::rewind(file);
                        wf->read_header();
                        headerSize = wf->get_header_size();
                        audioSize = wf->get_audio_size();
                        channels = wf->get_channels();
                        rate = wf->get_rate();
                        bitDepth = wf->get_bit_depth();
                        fileSize = audioSize + headerSize;
                        bytesPerSec = rate * channels * (bitDepth / 8);
                        blockSize = channels * (bitDepth / 8);
                        sampleSize = blockSize / channels;
                        totalBlocks = audioSize / blockSize;
                        peak = new Peak();
                        peak->build(this);
                        rewind();
                        prafFile=0;
                        wavFile=wf;
                        break;
                        }
                case OGGVORBIS:
                case MPEG3:
                default:
                        {
                        PERROR ("Format not supported inside Protux. Try Genpraf to convert it to PRAF format.");
                        }
                }
        PEXIT;
        return 1;
        }



int Audio::create(QString pRootDir, QString pFileName, int pChannels, int pRate, int pSize,
                        QString pProjectLabel, QString pGroupLabel,
                        QString pHardwareLabel, QString pDescription)
        {
        PENTER;
        rootDir = pRootDir;
        filename = pFileName;
        PMESG("Creating audio %d channels, %d Hz, %d bits",pChannels,pRate,pSize);
        //PMESG("Project : " << pProjectLabel);
        //PMESG("Group : " << pGroupLabel);
        //PMESG("Hardware : " << pHardwareLabel);
        //PMESG("Desc : " << pDescription);

        QString absoluteFilename = rootDir + "/" + filename;
        PrafFile *pf =  new PrafFile();
        if (pf->create((char*)absoluteFilename.latin1())<0)
                {
                delete pf;
                PERROR("Cannot create new file");
                return -1;
                }
        pf->write_header(pChannels, pRate, pSize, 0,
                         pProjectLabel, pGroupLabel,
                         pHardwareLabel, pDescription);

        headerSize = pf->get_header_size();
        audioSize = pf->get_audio_size();
        fileSize = audioSize + headerSize;
        channels = pf->get_channels();
        rate = pf->get_rate();
        bitDepth = pf->get_bit_depth();
        bytesPerSec = rate * channels * (bitDepth / 8);
        blockSize = channels * (bitDepth / 8);
        totalBlocks = audioSize / blockSize;
        prafFile=pf;
        wavFile=0;
        file = pf->get_file();
        PEXIT;
        return 1;
        }


int Audio::rebuild_peaks()
        {
        PENTER;
        if (peak)
                delete peak;
        peak = new Peak();
        peak->build(this);
        PEXIT;
        return 1;
        }


int Audio::unbuild()
        {
        PENTER;
        if (file!=NULL)
                if (fclose(file)<0)
                        PERROR("Cannot close file" );
        PEXIT;
        return 1;
        }



void Audio::rewind()
        {
        fseek(file, headerSize, SEEK_SET);
        }



void Audio::goto_end()
        {
        fseek(file, 0, SEEK_END);
        }


void Audio::locate( long long blockpos)
        {
        fseek(file, headerSize + (blockpos * blockSize), SEEK_SET);
        }


long long Audio::get_location()
        {
        long long p = (ftell(file) - headerSize) / blockSize;
        return p;
        }


FILE* Audio::get_file()
        {
        return file;
        }


long long Audio::get_file_size()
        {
        return fileSize;
        }

long long Audio::get_audio_size()
        {
        return audioSize;
        }

int Audio::get_block_size()
        {
        return blockSize;
        }

int Audio::get_header_size()
        {
        return headerSize;
        }

int Audio::get_rate()
        {
        return rate;
        }


int Audio::get_bit_depth()
        {
        return bitDepth;
        }


int Audio::get_channels()
        {
        return channels;
        }



QString Audio::get_filename()
        {
        return filename;
        }


void Audio::set_filename(QString fn)
        {
        filename = fn;
        }


void Audio::show_info()
        {
        /*PMESG("Audio info : ");
        PMESG("   headerSize  = " << headerSize);
        PMESG("   audioSize   = " << audioSize);
        PMESG("   channels    = " << channels);
        PMESG("   rate        = " << rate);
        PMESG("   bitDepth    = " << bitDepth);
        PMESG("   fileSize    = " << fileSize);
        PMESG("   bytesPerSec = " << bytesPerSec);
        PMESG("   blockSize   = " << blockSize);
        PMESG("   sampleSize  = " << sampleSize);
        PMESG("   totalBlocks = " << totalBlocks);
        */
        }

// IMPROVE NEXT TWO METHODS
// what is the difference between them ?
// Should it always get the total/last block from the phisical file ?
// should it store total blocks in a variable , or always calculate ir ?


long long Audio::get_last_block()
        {
        long tempPos = ftell(file);
        goto_end();
        long lastfilebyte = ftell(file);
        long lastaudiobyte = lastfilebyte - headerSize;
        long lastblock = lastaudiobyte / blockSize;
        fseek(file, tempPos, SEEK_SET);
        return lastblock;
        }

long long Audio::get_total_blocks()
        {
        return totalBlocks;
        }



int Audio::fix_size()
        {
        PENTER;
        if (prafFile)
                {
                PMESG("Fixing Praf Size");
                prafFile->fix_audio_size();
                audioSize=prafFile->get_audio_size();
                }
        else
        if (wavFile)
                {
                PMESG("Fixing Audio Size");
                wavFile->fix_audio_size();
                audioSize=wavFile->get_audio_size();
                }
        totalBlocks = audioSize / blockSize;                
        PEXIT;
        return 1;
        }


// eof
