/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#ifndef AUDIO_HH
#define AUDIO_HH

#include <stdio.h>
#include <qstring.h>


class Song;
class Peak;
class PrafFile;
class WavFile;

class Audio
    {
    public :
        static const int PRAF=1;
        static const int OGGVORBIS=2;
        static const int WAV=3;
        static const int MPEG3=4;
        Peak* peak;

        Audio(Song* pParentSong);
        ~Audio();
        int build(QString rootDir, QString pFileName);
        int create(QString rootDir, QString pFileName, int pChannels, int pRate, int pSize,
                QString pProjectLabel, QString pGroupLabel,
                QString pHardwareLabel, QString pDescription);
        int rebuild_peaks();
        int unbuild();
        void locate(long long blockpos);
        long long get_location();
        void rewind();
        void goto_end();
        long long get_current_block();
        long long get_file_size();
        long long get_audio_size();
        long long get_total_blocks();
        long long get_last_block();
        int get_block_size();
        int get_rate();
        int get_bit_depth();
        int get_channels();
        int get_header_size();
        QString get_filename();
        void set_filename(QString fn);
        FILE* get_file();
        int fix_size();
        void show_info();

    private:

        PrafFile* prafFile;
        WavFile* wavFile;
        Song* parentSong;

        int audioType;
        FILE* file;
        QString rootDir;
        QString filename;
        int file_status;
        int channels;
        int rate;
        int bitDepth;
        int blockSize;
        int sampleSize; // in bytes
        long long fileSize;
        long long headerSize;
        long long audioSize;
        long long bytesPerSec;
        long long totalBlocks;
        long long filePos;
        int fix_header();

        friend class Peak;
    };

#include "Song.hh"
#include "Peak.hh"
#include "PrafFile.hh"
#include "WavFile.hh"
#endif


