/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include <qcolor.h>

#include <mustux.h>

#include "BusMonitor.hh"
#include <stdlib.h>

// This is for VU : db = 20 * log ( sample / MaxSample )

static const int MAX_LEVEL = 36;
static const int BASEX = 800;
static const int BASEY = 100;
static const int HEIGHT = 90;
static const int WIDTH = 50;
static const int SPACING = 10;

#define BG_COLOR QColor(140,160,120)
#define TEXT_COLOR Qt::black
//#define LEVEL_CLEAR_COLOR QColor(20,20,20)
#define LEVEL_CLEAR_COLOR QColor(60,70,60)



BusMonitor::BusMonitor(MustuxLcd* pParentLcd)
        {
        PENTERCONS;
        int nbuses = MustuxAudioDeviceMapper::get_number_of_buses();
        parentLcd = pParentLcd;
        nbuses = nbuses;
        lCurrLevel = new int[nbuses];
        rCurrLevel = new int[nbuses];
        for (int i=0; i < nbuses; i++)
                {
                lCurrLevel[i] = 0;
                rCurrLevel[i] = 0;
                }

        updateTimer = new QTimer(this);
        connect( updateTimer , SIGNAL(timeout()), this, SLOT(update()));
// FOR LATER       spectrogram=0;
        PEXITCONS;
        }


BusMonitor::~BusMonitor()
        {
        PENTERDES;
        delete updateTimer;
        delete lCurrLevel;
        delete rCurrLevel;
        PEXITDES;
        }





void BusMonitor::clear()
        {
        QPainter* p = parentLcd->painter;
        int nbuses = MustuxAudioDeviceMapper::get_number_of_buses();
        p->setPen(QColor(20,20,45));
        p->setFont(QFont("Fixed",6));
        for (int b=0; b<nbuses; b++)
                {

                QString si; si.setNum(b);
                QString bn = MustuxAudioDeviceMapper::get_bus_name(b);
                bn = bn.left(bn.find('-'));
                QString t = si+"-"+bn;
                int bx = BASEX + b * (WIDTH + SPACING);

                p->setPen(TEXT_COLOR);
		//                p->drawRect(bx,BASEY-HEIGHT,WIDTH,HEIGHT);  // gets anyway overriden by next line
		p->fillRect(bx,BASEY-HEIGHT,WIDTH,HEIGHT,BG_COLOR);
                p->drawText(bx,BASEY - 11, WIDTH, 6, Qt::AlignCenter | Qt::DontClip, t);
                p->setPen(LEVEL_CLEAR_COLOR);
                for (int i=0; i<MAX_LEVEL; i++)
                        {
                        int by = BASEY - 15 - i * 2;
                        p->drawLine(bx + 9,
                                    by,
                                    bx + 10 + 10,
                                    by);
                        p->drawLine(bx + 9 + 20,
                                    by,
                                    bx + 10 + 20 + 10,
                                    by);
                        }
                }
        //MAYBE USEFULL drawArea->update(25,BASEY+32, 150, 50);
        }




void BusMonitor::activate()
        {
        clear();
        updateTimer->start(40);
        isActive=true;
//        if (spectrogram) spectrogram->activate();
        }


void BusMonitor::deactivate()
        {
        isActive=false;
        updateTimer->stop();
        //if (spectrogram) spectrogram->deactivate();
        }


bool BusMonitor::is_active()
        {
        return isActive;
        }




void BusMonitor::update_levels(int bus, char* buf, int bufSize)
        {
        // FIX ME : Im am assuming that buf is always stereo. VERY DANGEROUS
        int channels =2;
        char* p  = buf;
        int maxL=0;
        int maxR=0;
        short sho;
        int totalsamples = bufSize/(channels*sizeof(short));
        for (int i=0; i <totalsamples; i++)
                {
                sho = (short) *(short*)p;
                if (abs(sho) > maxL) maxL = (int)abs(sho);
                p+=sizeof(short);
                sho = (short) *(short*)p;
                if (abs(sho) > maxR) maxR = (int)abs(sho);
                p+=sizeof(short);
                }
        float fl = (float) maxL / 32768;
        float fr = (float) maxR / 32768;
        lCurrLevel[bus] = (int) (fl*MAX_LEVEL);
        rCurrLevel[bus] = (int) (fr*MAX_LEVEL);
        // FOR LATER
        /*
        if (spectrogram) // FOR LATER && (spectrogram->get_route_in()==bus)
        spectrogram->set_data((short*)bufPointer);
        */
        }


void BusMonitor::update()
        {
        int nbuses = MustuxAudioDeviceMapper::get_number_of_buses();
        QPainter* p = parentLcd->fastPainter;
        
        for (int b = 0; b < nbuses; b++)
                {
                int bx = BASEX + b * (WIDTH + SPACING);

                // First clear where is not used
                p->setPen(LEVEL_CLEAR_COLOR);
                for (int i=lCurrLevel[b]; i<MAX_LEVEL; i++)
                        {
                        int by = BASEY - 15 - i * 2;
                        p->drawLine(bx + 9,
                                    by,
                                    bx + 10 + 10,
                                    by);
                        }
                for (int i=rCurrLevel[b]; i<MAX_LEVEL; i++)
                        {
                        int by = BASEY - 15 - i * 2;
                        p->drawLine(bx + 9 + 20,
                                    by,
                                    bx + 10 + 20 + 10,
                                    by);
                        }

                // check overload
                /*
                if (lCurrLevel[b]>=MAX_LEVEL)
                        {
                        p->fillRect( BASEX + MAX_LEVEL + 5, BASEY+32, 26,9,QColor(250,0,0));
                        p->setPen(QColor(0,0,0));
                        p->setFont( QFont( "Helvetica", 6) );
                        p->drawText( BASEX + MAX_LEVEL + 10, BASEY+39, "OVR");
                        }
                if (rCurrLevel[b]>=MAX_LEVEL)
                        {
                        p->fillRect( BASEX + MAX_LEVEL + 5, BASEY+42, 26, 9, QColor(250,0,0) );
                        p->setPen( QColor(0,0,0) );
                        p->setFont( QFont( "Helvetica", 6) );
                        p->drawText( BASEX + MAX_LEVEL + 10, BASEY+49, "OVR");
                        }
                */
                
                // draw the peak meter
                for (int i = 0; i < lCurrLevel[b]; i++)
                        {
//                      p->setPen(QColor( i*2,255-(i>(MAX_LEVEL/2)?(i-(MAX_LEVEL/2))*3:0),0) );
			p->setPen(QColor(i > (MAX_LEVEL / 2) ? (510 / MAX_LEVEL) * i - 255 : 0,
					 255 - (i > (MAX_LEVEL / 2) ? (510 / MAX_LEVEL) * i - 255 : 0),
					 0));
                        int by = BASEY - 15 - i * 2;
                        p->drawLine(bx + 9,
                                    by,
                                    bx + 10 + 10,
                                    by);
                        }
                for (int i = 0; i < rCurrLevel[b]; i++)
                        {
			p->setPen(QColor(i > (MAX_LEVEL / 2) ? (510 / MAX_LEVEL) * i - 255 : 0,
					 255 - (i > (MAX_LEVEL / 2) ? (510 / MAX_LEVEL) * i - 255 : 0),
					 0));
                        int by = BASEY - 15 - i * 2;
                        p->drawLine(bx + 9 + 20,
                                    by,
                                    bx + 10 + 20 + 10,
                                    by);
                        }

                //MAYBE USEFULL drawArea->update(BASEX,BASEY+32, MAX_LEVEL + 50,20);
                }
        }

/*void BusMonitor::set_output_vu(Spectrogram* pSpectrogram)
        {
        spectrogram = pSpectrogram;
        }
*/


// eof

