/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include "CursorManager.hh"


static const int LOCATION_INFO_X = 510;
static const int LOCATION_INFO_Y = 60;
static const int LOCATION_INFO_W = 200;
static const int LOCATION_INFO_H = 30;


CursorManager::CursorManager(Song* pAssocSong)
        {
        PENTERCONS;
        assocSong = pAssocSong;
        drawArea = assocSong->get_tracks_area();
        defaultLcd = assocSong->get_default_lcd();
        isFloatCursorHidden = false;
        isWorkCursorHidden = false;
        isDragCursorHidden = true;
        floatX=0;
        playX=0;
        workX=0;
        mode=MODE_NOTDEFINED;
        isEnabled=false;
        followPlayTimer = new QTimer(this);
        connect( followPlayTimer , SIGNAL(timeout()), this, SLOT(update_play_cursor()));

	workCursorColor = new QColor(250,50,50);
        PEXITCONS;
        }


CursorManager::~CursorManager()
        {
        PENTERDES;
        delete followPlayTimer;
        PEXITDES;
        }



void CursorManager::start()
        {
        PENTER;
        set_mode(MODE_FLOAT);
        show_work_cursor();
        drawArea->setMouseTracking(true);
        enable();
        PEXIT;
        }


void CursorManager::set_mode(int pMode, int pOrigX, int pOrigY,
              int pCursorW, int pCursorH,
              int pOffsetX, int pOffsetY)
        {
        mode = pMode;
        xOrig = pOrigX;
        yOrig = pOrigY;
        xOffset = pOffsetX;
        yOffset = pOffsetY;
        cursorW = pCursorW;
        cursorH = pCursorH;
        }



void CursorManager::enable()
        {
        if (!isEnabled)
                {
                isEnabled=true;
                show_work_cursor();
                switch (mode)
                        {
                        case MODE_NONE:
                                {
                                return;
                                }
                        case MODE_FLOAT:
                                {
                                show_float_cursor();
                                break;
                                }
                        case MODE_DRAG:
                                {
                                show_drag_cursor();
                                break;
                                }
                        case MODE_PLAY:
                                {
                                show_play_cursor();
                                break;
                                }
                        case MODE_NOTDEFINED:
                                {
                                break;
                                }
                        }
                }
        }


void CursorManager::disable()
        {
        if (isEnabled)
                {
                isEnabled=false;

                hide_work_cursor();
                switch (mode)
                        {
                        case MODE_FLOAT:
                                {
                                hide_float_cursor();
                                break;
                                }
                        case MODE_DRAG:
                                {
                                hide_drag_cursor();
                                break;
                                }
                        case MODE_PLAY:
                                {
                                hide_play_cursor();
                                break;
                                }
                        case MODE_NOTDEFINED:
                                {
                                break;
                                }
                        }
                }
        }


void CursorManager::update(int x, int y)
        {
        switch (mode)
                {
                case MODE_FLOAT:
                        {
                        update_float_cursor(x);
                        break;
                        }
                case MODE_DRAG:
                        {
                        update_drag_cursor(x,y);
                        break;
                        }
                case MODE_PLAY:
                        {
                        update_play_cursor();
                        break;
                        }
                case MODE_NOTDEFINED:
                        {
                        break;
                        }
                }

        QPainter* painter = defaultLcd->painter;
        defaultLcd->clear(LOCATION_INFO_X, LOCATION_INFO_Y, LOCATION_INFO_W , LOCATION_INFO_H);
        long long bpos = assocSong->xpos_to_block(x);
        if (bpos<0) bpos=0;

        float f =  44100 / 30;
        float flo = (float) bpos /  f;
        long long framepos = ( long long ) flo;
        int pfr = (int) framepos % 30;
        int pho = (int) (framepos/108000) % 60;
        int pmi = (int) (framepos/1800) % 60;
        int pse = (int) (framepos/30) % 60;

        if (pho<0) pho=0;
        if (pmi<0) pmi=0;
        if (pse<0) pse=0;
        if (pfr<0) pfr=0;
        QString spho; spho.setNum(pho);
        QString spmi; spmi.setNum(pmi); if (pmi<10) spmi.prepend("0");
        QString spse; spse.setNum(pse); if (pse<10) spse.prepend("0");
        QString spfr; spfr.setNum(pfr); if (pfr<10) spfr.prepend("0");
        QString spos = spho + ":" + spmi + ":" + spse + "," + spfr;

        painter->setPen( QColor(0,0,0));
        painter->setFont( QFont( "Fixed", 10));
        painter->drawText(LOCATION_INFO_X, LOCATION_INFO_Y+15, spos);

        QString sbpos; sbpos.setNum((double)bpos,'f',0);
        painter->drawText(LOCATION_INFO_X + 90, LOCATION_INFO_Y+15, " : " + sbpos  );

        defaultLcd->update(LOCATION_INFO_X,LOCATION_INFO_Y,LOCATION_INFO_W,LOCATION_INFO_H);
        }


int CursorManager::get_orig_x()
        {
        return xOrig;
        }

int CursorManager::get_orig_y()
        {
        return yOrig;
        }


int CursorManager::get_x_offset()
        {
        return xOffset;
        }

int CursorManager::get_y_offset()
        {
        return yOffset;
        }

void CursorManager::set_x_offset(int x)
        {
        xOffset=x;
        }

void CursorManager::set_y_offset(int y)
        {
        yOffset=y;
        }

void CursorManager::set_work_at(int x)
        {
        PENTER;
        update_work_cursor(x);
        PEXIT;
        }


// PROTECTED



void CursorManager::show_drag_cursor()
        {
        if (isDragCursorHidden)
                {
                toggle_drag_cursor();
                isDragCursorHidden = false;
                }
        }


void CursorManager::hide_drag_cursor()
        {
        if (!isDragCursorHidden)
                {
                toggle_drag_cursor();
                isDragCursorHidden = true;
                }
        }


void CursorManager::show_float_cursor()
        {
        if (isFloatCursorHidden)
                {
                toggle_float_cursor(floatX, drawArea->height());
                isFloatCursorHidden = false;
                }
        }


void CursorManager::hide_float_cursor()
        {
        if (!isFloatCursorHidden)
                {
                toggle_float_cursor(floatX, drawArea->height());
                isFloatCursorHidden = true;
                }
        }


void CursorManager::show_work_cursor()
        {
        if (isWorkCursorHidden)
                {
                toggle_work_cursor();
                isWorkCursorHidden = false;
                }
        }


void CursorManager::hide_work_cursor()
        {
        if (!isWorkCursorHidden)
                {
                toggle_work_cursor();
                isWorkCursorHidden = true;
                }
        }


void CursorManager::show_play_cursor()
        {
        toggle_play_cursor(playX, drawArea->height());
        followPlayTimer->start(25);//GlobalProperties::get_int("PLAY_CURSOR_REFRESH_RATE");
        }


void CursorManager::hide_play_cursor()
        {
        followPlayTimer->stop();
        toggle_play_cursor(playX, drawArea->height());
        }



void CursorManager::update_float_cursor(int x)
        {
        toggle_float_cursor(floatX, drawArea->height());
        floatX = x;
        toggle_float_cursor(floatX, drawArea->height());
        }



void CursorManager::update_drag_cursor(int x, int y)
        {
        toggle_drag_cursor();
        floatX = x;
        floatY = y;
        toggle_drag_cursor();
        }


void CursorManager::update_work_cursor(int x)
        {
        toggle_work_cursor();
        workX= x;
        toggle_work_cursor();
        }


void CursorManager::update_play_cursor()
        {
        if (assocSong->get_status()==Song::STOPPED)
                {
                disable();
                set_mode(CursorManager::MODE_FLOAT);
                enable();
                }
        else
                {
                int fullW = drawArea->width();
                int fullH = drawArea->height();
                int x = assocSong->get_playing_xpos();
                if ( (x >= 0 ) && ( x <= fullW ) )
                        {
                        toggle_play_cursor(playX, fullH);
                        playX = x;
                        toggle_play_cursor(playX, fullH);
                        }
                else if ( x > fullW )
                        {
                        //playX = fullW - 1;
                        assocSong->scroll_right( fullW);
                        }
                }
        }


void CursorManager::toggle_play_cursor(int x, int h)
        {
        drawArea->painter->setRasterOp(Qt::XorROP);
        drawArea->painter->setPen(QColor(255,0,255));
        drawArea->painter->drawLine( x  , 0, x  , h);
        drawArea->painter->setRasterOp(Qt::CopyROP);
        drawArea->update(x  - 1, 0, 2,-1);
        }



void CursorManager::toggle_float_cursor(int x, int h)
        {
        drawArea->painter->setRasterOp(Qt::XorROP);
        drawArea->painter->setPen(QColor(150,155,0));
        drawArea->painter->drawLine( x , 0, x, h);
        drawArea->painter->setRasterOp(Qt::CopyROP);
        drawArea->update(x,0,2,-1);
        }


void CursorManager::toggle_drag_cursor()
        {
        drawArea->painter->setRasterOp(Qt::XorROP);
        drawArea->painter->setPen(QColor(155,105,155));
        int x0 = floatX - xOffset;
        int x1 = x0 + cursorW;
        int y0 = floatY - yOffset;
        int xc = floatX;
        int fullH = drawArea->height();
        drawArea->painter->drawRect( x0 + 1,  y0, cursorW - 1, cursorH );
        drawArea->painter->drawLine( x0, 0, x0, fullH);
        drawArea->painter->drawLine( xc, 0, xc, fullH);
        drawArea->painter->drawLine( x1, 0, x1, fullH);
        drawArea->painter->setRasterOp(Qt::CopyROP);
        drawArea->update(x0, floatY  - yOffset, cursorW + 1, cursorH + 1 );
        drawArea->update(x0 - 1 , 0 , 1, fullH);
        drawArea->update(x1 - 1 , 0 , 1, fullH);
        }


void CursorManager::toggle_work_cursor()
        {
        drawArea->painter->setRasterOp(Qt::XorROP);
        drawArea->painter->setPen(*workCursorColor);
        int h = drawArea->height();
        int y=0;
        while (y<h)
                {
                drawArea->painter->drawLine( workX, y, workX, y+5);
                y+=10;
                }
        drawArea->painter->setRasterOp(Qt::CopyROP);
        drawArea->update(workX-1,0,2,-1);
        }

void CursorManager::set_work_cursor_color(int r, int g, int b)
        {
	workCursorColor->setRgb(r,g,b);
        }

// eof

