/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#ifndef CURSORMANAGER_HH
#define CURSORMANAGER_HH

#include <mustux.h>
#include <qwidget.h>
#include <qpainter.h>

class Song;

class CursorManager : public MustuxEngine
        {
        Q_OBJECT
        public :
                CursorManager(Song* pAssocSong);
                ~CursorManager();
                static const int MODE_NOTDEFINED=0;
                static const int MODE_NONE=1;
                static const int MODE_FLOAT=2;
                static const int MODE_DRAG=3;
                static const int MODE_PLAY=4;


                void set_mode(int mode, int pOrigX=0, int pOrigY=0,
                        int pCursorW=1, int pCursorH=-1,
                        int pOffsetX=0, int pOffsetY=0);

                void start();

                void update(int x, int y);
                void enable();
                void disable();
                void set_work_at(int x);

                int get_orig_x();
                int get_orig_y();
                int get_x_offset();
                int get_y_offset();
                void set_x_offset(int x);
                void set_y_offset(int y);
		
		void set_work_cursor_color(int r, int g, int b);

        protected :
		QColor* workCursorColor;

                void show_float_cursor();
                void show_work_cursor();
                void show_drag_cursor();
                void show_play_cursor();
                void update_work_cursor(int x);
                void update_drag_cursor(int x, int y);
                void update_float_cursor(int x);
                void hide_float_cursor();
                void hide_work_cursor();
                void hide_drag_cursor();
                void hide_play_cursor();

                Song* assocSong;
                MustuxDrawable *drawArea;
                QTimer* followPlayTimer;

                int mode;
                int floatX;
                int floatY;
                int xOrig;
                int yOrig;
                int xOffset;
                int yOffset;
                int cursorW;
                int cursorH;
                int workX;
                int playX;

                bool isEnabled;

                bool isFloatCursorHidden;
                bool isWorkCursorHidden;
                bool isDragCursorHidden;

                void toggle_work_cursor();
                void toggle_float_cursor(int x, int h);
                void toggle_drag_cursor();
                void toggle_play_cursor(int x, int h); // vai virar generico

                MustuxLcd* defaultLcd;
                QPainter* painter;

        public slots:
                void update_play_cursor();

        private:
                friend class Song;
        };

#include "Song.hh"

#endif



