/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include "Interface.hh"
#include "ProjectManager.hh"
#include "ColorManager.hh"

static const int MINIMUM_INTERFACE_WIDTH = 600;
static const int MINIMUM_INTERFACE_HEIGHT = 300;

Interface::Interface(QWidget *parent) : MustuxInterface( parent )
        {
        PENTERCONS;

        resize(MINIMUM_INTERFACE_WIDTH,MINIMUM_INTERFACE_HEIGHT);
        setMinimumSize(MINIMUM_INTERFACE_WIDTH,MINIMUM_INTERFACE_HEIGHT);
        setIcon(QPixmap (GlobalProperties::RESOURCES_DIR+"/protux.xpm") );

        ColorManager::init();

        // top level container
        mainVBoxLayout = new QVBoxLayout(this, 6);
        mainVBoxLayout->setResizeMode(QLayout::FreeResize);

                QWidget* hbox = new QWidget( this );
                hbox->setFixedHeight(100);  //TODO : Must become a Global Properties property
                QHBoxLayout* hboxlayout = new QHBoxLayout( hbox );

                        infoLcd = new MustuxLcd(200, 100, hbox , "Main LCD");  //TODO : Must become a Global Properties property
                        // infoLcd->show();
                        // infoLcd->setFrameStyle( QFrame::Panel | QFrame::Plain );
                        hboxlayout->addWidget(infoLcd);
                        set_default_lcd(infoLcd);
                        spectrogram = (Spectrogram*) 0;
/*
                        spectrogram = new Spectrogram( GlobalProperties::get_int("SPECTROGRAM_WIDTH"),
                                                       GlobalProperties::get_int("SPECTROGRAM_HEIGHT"),
                                                       hbox, "Spectrogram");
                        spectrogram ->setFixedWidth(GlobalProperties::get_int("SPECTROGRAM_WIDTH"));//TOGP
                        //spectrogram->show();
                        //spectrogram->setFrameStyle( QFrame::Panel | QFrame::Plain );
                        hboxlayout->addWidget(spectrogram);
*/

        mainVBoxLayout->addWidget( hbox );

        busMonitor = new BusMonitor(infoLcd);

        // FLYING WINDOWS ----------------------
        projectManager = new ProjectManager(this);

        // Try to load the "Untitled" project from a previous session. If it cant, it creates a new one with 2 songs
	if (projectManager->projectExists("Untitled"))
	        projectManager->load_project("Untitled");
	else
	        projectManager->create_new_project("Untitled", 2);

        helpWindow = new Help(GlobalProperties::RESOURCES_DIR+"/help.text", (QWidget*) 0);

        QString msg = "Welcome to Protux ";
        msg.append(GlobalProperties::get("VERSION"));
        msg.append("      Press H for Quick Help                     ");
        infoLcd->roll_message((const char*) msg.latin1());

        jmb->init(GlobalProperties::RESOURCES_DIR+"/jmb.map");// FOR LATER SettingsManager::get_property("JMB_MAP_FILE");

        isHelpHidden=true;

        globalPropertiesWindow = new GlobalPropertiesWindow(this);

        mainVBoxLayout->activate();
        showMaximized();

        tuner = (MustuxTuner*) 0;

        show_splash_screen();


        PEXITCONS;
        }


Interface::~Interface()
        {
        PENTERDES;
        delete globalPropertiesWindow;
        delete busMonitor;
        delete helpWindow;
        delete projectManager;
        PEXITDES;
        }


int Interface::process_jmb_action(MustuxJogMouseBoardMessage* jmbMesg)
        {
        int r=-1;
        int action = jmbMesg->action;
        // bool isBeginning = jmbMesg->moment; UNCOMMENT ME, BECAUSE isBeginning will be be very usefull
        QString actionName = jmbMesg->label;
        int parameter = jmbMesg->parameter;
        int collNumber = jmbMesg->collectedNumber;

        // Interface scope actions
        if (actionName == "QUIT")
                {
                if (currentProject)
                        currentProject->save();
                QApplication::exit();
                }
        else if (actionName == "ABOUT")
                {
                r=1;
                }
        else if (actionName == "FILE STUFF")
                {
                projectManager->show();
		projectManager->raise();
                r=1;
                }
        else if (actionName == "LOCK/UNLOCK EVERYTHING")
                {
                if (jmb->is_locked())
                        jmb->unlock();
                else
                        jmb->lock();
                r=1;
                }

        else if (actionName == "HELP")
                {
                help();
                r=1;
                }

        else if (actionName == "TUNER")
                {
                if (tuner)
                        {
                        delete tuner;
                        tuner = (MustuxTuner*) 0;
                        }
                else
                        {
                        tuner = new MustuxTuner(1,true);
                        tuner->init_frontend();
                        tuner->show();
                        }

                r=1;
                }

        else if (actionName == "GLOBAL PROPERTIES")
                {
                globalPropertiesWindow->show();
                r=1;
                }
        else
                {
                if (currentProject)
                        {
                        // PROJECT SCOPE ACTIONS
                        if (actionName == "SAVE CURRENT PROJECT")
                                {
                                r = currentProject->save();
                                }
                        else if (actionName == "RENDER SONG")
                                {
                                r = currentProject->get_current_song()->render();
                                }
                        else
                                {
                                // SONG SCOPE ACTIONS
                                Song* currentSong = currentProject->get_current_song();
                                r = currentSong->process_jmb_action(jmbMesg);
                                }
                        }
                else
                        {
                        info("OPEN OR CREATE A PROJECT FIRST!!");
                        r=-1;
                        }
                }
        return r;
        }




void Interface::show_splash_screen()
        {
        splashScreen = new SplashScreen(GlobalProperties::RESOURCES_DIR+"/protuxSplash.jpg");
        }


void Interface::hide_splash_screen() // deprecated
        {
        }


void Interface::help()
        {
        PENTER;
        if (isHelpHidden)
                {
                helpWindow->show();
                QString authors="The Protux Team : Luciano Giordana, Reinhard Amersberger, Tapio Laxstrom, David Bain, Nathan Langford, Fabio dos Santos, Martin Herren, Eduardo Pimenta, Valery Beluntsov, Marcio Enrico, and all the people from Free Software world, who made important technologies where Protux is based on ( Gcc, QT, KDE, XFree, Linux, and so on...)                                              ";
                infoLcd->roll_message(authors,80,2);
                isHelpHidden=false;
                }
        else
                {
                helpWindow->close();
                isHelpHidden=true;
                }
        PEXIT;
        }

void Interface::quick_help()
        {
        PENTER;
        PEXIT;
        }

void Interface::show_jmb_map()
        {
        PENTER;
        PEXIT;
        }

void Interface::about_protux()
        {
        PENTER;
        PEXIT;
        }

MustuxLcd* Interface::get_default_lcd()
        {
        return infoLcd;
        }


void Interface::update_properties()
        {
        jmb->set_clear_time(GlobalProperties::get_int("JMB_CLEAR_TIME"));
        jmb->set_hold_sensitiveness(GlobalProperties::get_int("JMB_HOLD_SENSITIVENESS_FACTOR"));
        jmb->set_double_press_interval(GlobalProperties::get_int("JMB_DOUBLE_PRESS_INTERVAL"));
        }


void Interface::info(QString message)
        {
        defaultLcd->print(message,0);
        }

void Interface::info(QString message, int line)
        {
        defaultLcd->print(message,line);
        }

BusMonitor* Interface::get_bus_monitor()
        {
        return busMonitor;
        }


void Interface::set_current_project(Project *pCurrentProject)
        {
	PENTER;
        currentProject = pCurrentProject;
        mainVBoxLayout->addWidget(currentProject);
        mainVBoxLayout->activate();
	currentProject->set_output_spectrogram(spectrogram);
        currentProject->show();
	PEXIT;
        }


// These methods are here just to avoid the annoying warnings suck  "QObject::connect: No such
// slot MultiTrackArea::cleanupEventFilter()" or "connect : No such signal Interface::destroyed()"
// when running protux. If anybody knows how to avoid this without implementing this method,
// please, send an email to one of protux developers. Thanks a lot.


void Interface::cleanupEventFilter()
        {
        }

/*void Interface::destroyed()
        {
        }
*/

// EOF

