/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#ifndef INTERFACE_HH
#define INTERFACE_HH


#include <qlayout.h>
#include <mustux.h>
#include "GlobalProperties.hh"
#include "GlobalPropertiesWindow.hh"
#include <qimage.h>

class SplashScreen;
class Help;
class GlobalPropertiesWindow;
class Spectrogram;
class ProjectManager;
class Project;
class Song;
class BusMonitor;


class Interface : public MustuxInterface
        {
        Q_OBJECT

        public :
                Interface(QWidget *parent);
                ~Interface();

                int process_jmb_action( MustuxJogMouseBoardMessage* jmbMesg );
                void update_properties();
                int new_project();
                int import_audio(const QString &filename);

	        void set_current_project(Project *pCurrentProject);

                MustuxLcd* get_default_lcd();
                BusMonitor* get_bus_monitor();

                void info(QString message);
                void info(QString message, int line);


        protected :
                Project* currentProject;
                MustuxLcd* infoLcd;
                MustuxDrawable* projectArea;
                MustuxTuner* tuner;

                Help* helpWindow;
                ProjectManager* projectManager;
                GlobalPropertiesWindow* globalPropertiesWindow;
                Spectrogram* spectrogram;
                SplashScreen* splashScreen;

                bool isHelpHidden;
                void show_project_info();
                
                QImage* splash;

        private:
                QTimer* splashTimer;
	        QVBoxLayout *mainVBoxLayout;

        public slots :
                void help();
                void quick_help();
                void show_jmb_map();
                void about_protux();
                void show_splash_screen();
                void hide_splash_screen();
                void cleanupEventFilter(); // Just to avoid the warning. See the comment in the implementation

                BusMonitor* busMonitor;

        //public signals :
//                void destroyed();

        };

#include "SplashScreen.hh"
#include "Help.hh"
#include "GlobalPropertiesWindow.hh"
#include "Spectrogram.hh"
#include "ProjectManager.hh"
#include "Project.hh"
#include "Song.hh"
#include "BusMonitor.hh"

#endif


// eof
