/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#ifndef MIXER_HH
#define MIXER_HH

#include <mustux.h>
#include <qtimer.h>
#include <qthread.h>

class Audio;
class Song;
class Spectrogram;
class SubClip;
class Slice;
class Mixer;
class BusMonitor;


class ThPlayLoop : public QThread
        {
        public:
                ThPlayLoop(Mixer* pMixer);
                void run();
        private:
                Mixer* mixer;
        };

// MAYBE MIXER SHOULD BECOME A NAMESPACE...

class Mixer : public MustuxEngine
        {
        public:
                Mixer(Song* pSong);
                ~Mixer();

                int render_all(QString filename);
                int build_slice_list(int lastBlockPos);
                int play(long long startBlock);
                int force_stop();
                int stop();
                void show_slice_list();
                long long get_current_playing_block();
                char* get_current_mixed_audio_frag();
                int get_current_mixed_audio_frag_size();

                bool is_playing();
                bool next_slice();

                int get_slice_width_in_bytes();
                int get_slice_width_factor();
                int get_slice_width();

                Song* get_parent_song();
                
                int get_max_channels();

        protected :
                void play_loop();

                Song* song;

                Slice* firstSlice;
                Slice* currentSlice;
                int fragSize;
                int relativeBlockInsideSliceBeingPlayed;
                char* bufPointer;
                char* endbufPointer;
                bool isPlaying;
                bool stoppedByUser;
                bool* busStatus;
                long long playStartBlock;
                long long blockOffset;
                BusMonitor* busMonitor;


                MustuxLcd* lcd;

                ThPlayLoop* thPlayLoop;

                // mix buffers (per bus)
                char* mixBuffer[MustuxAudioDeviceMapper::MAX_BUSES];
                int mixBufferSize[MustuxAudioDeviceMapper::MAX_BUSES];

                // render buffer (for the whole slice)
                char* renderBuffer;
                int renderBufferSize;

                int sliceWidthFactor;
                int sliceWidth;
                int sliceWidthInBytes;
                int maxChannels;
        };


#include "Audio.hh"
#include "Song.hh"
#include "Spectrogram.hh"
#include "SubClip.hh"
#include "Slice.hh"
#include "BusMonitor.hh"


#endif


