/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include <stdio.h>
#include <math.h>
#include <qprogressdialog.h>
#include <qwidget.h>
#include <mustux.h>
#include "Audio.hh"
#include "Peak.hh"
#include <qtimer.h>


const int Peak::MAX_ZOOM_USING_SOURCEFILE = 8;
const int Peak::zoomStep[MAX_ZOOM_LEVELS] = {
                                                  1,    2,    4,     6,     8,    12,    16,    20,
                                                 24,   32,   48,    64,    64,    96,   128,   192,
                                                256,  384,  512,   768,  1024,  1536,  2048,  3072,
                                               4096, 6144, 8192, 12288, 16384, 24576, 32768, 65536
                                            };


Peak::Peak()
        {
        PENTERCONS;
        PEXITCONS;
        }


Peak::~Peak()
        {
        PENTERDES;
        PEXITDES;
        }



int Peak::build(Audio* anAudioSource)
        {
        PENTER;
        audioSource = anAudioSource;
        audioSource->rewind();
        audioSourceFile = audioSource->get_file();
        audioSourceFileSize = audioSource->get_file_size();
        audioSize = audioSource->get_audio_size();
        headerSize = audioSource->get_header_size();
        blockSize = audioSource->get_block_size();
        channels = audioSource->get_channels();
        rate = audioSource->get_rate();
        bitDepth = audioSource->get_bit_depth();
        static const int MAX_CHANNELS=2; // FIX ME
        short  lastRegionMax[MAX_ZOOM_LEVELS][MAX_CHANNELS];
        short  lastRegionMin[MAX_ZOOM_LEVELS][MAX_CHANNELS];
        long long posInLevel [ MAX_ZOOM_LEVELS ];
        long long totalAudioBlocks = (audioSize/blockSize);

        for (int hzoom = 0; hzoom<MAX_ZOOM_LEVELS; hzoom++)
                {
                RAMBuffer[hzoom] = (short*) 0;
                RAMBufferSize[hzoom]=0;
                }

        for (int hzoom=MAX_ZOOM_USING_SOURCEFILE+1;   hzoom<MAX_ZOOM_LEVELS;  hzoom++)
                {
                long long howManyZoomStepsForThisZoomLevel = totalAudioBlocks / zoomStep[hzoom];
                RAMBufferSize[hzoom] = (howManyZoomStepsForThisZoomLevel + 1) * channels;
                RAMBuffer[hzoom] = new short[RAMBufferSize[hzoom]];
                //PMESG("Created RAM buffer for HZOOM = %d of %d shorts ", hzoom , RAMBufferSize[hzoom] );
                }

        // try to load a previously saved image before rescan the file...
        if (load()>0)
                {
                PMESG("Loaded peaks from previously saved image");
                PEXIT;
                return 1;
                }

        // No saved peak. So it needs to re-scan the file
        PMESG("No previously saved peak image found. Rescanning file...");
        for (int hzoom=MAX_ZOOM_USING_SOURCEFILE+1;   hzoom<MAX_ZOOM_LEVELS;  hzoom++)
                {
                for (int ch=0; ch<channels; ch++)
                        {
                        lastRegionMax[hzoom][ch]=0;
                        lastRegionMin[hzoom][ch]=0;
                        }
                posInLevel[hzoom]=0;
                }

        QString afn = audioSource->get_filename();
        short sho;
        long long blockPos = 0;
        MustuxLcd* lcd = audioSource->parentSong->get_default_lcd();
        lcd->print("Rescanning Peaks :      %", 3);
        while (!feof(audioSourceFile))
                {
                if (blockPos % 50000==0)
                        {
                        float f = 100.0 * (float) blockPos / totalAudioBlocks;
                        int i = (int) f;
                        QString si; si.setNum(i);
                        lcd->print(si , 3,  20, false);
                        }
                // Reads the whole block, and if its number is a key point for some
                // zoom level, use its data to fill the buffers
                for  (int ch=0; ch<channels; ch++)
                        {
                        fread(&sho, 1, sizeof(short), audioSourceFile);
                        for (int hzoom=MAX_ZOOM_USING_SOURCEFILE+1; hzoom<MAX_ZOOM_LEVELS; hzoom++)
                                {
                                if (sho>lastRegionMax[hzoom][ch]) lastRegionMax[hzoom][ch]=sho;
                                if (sho<lastRegionMin[hzoom][ch]) lastRegionMin[hzoom][ch]=sho;
                                }

                        // checks if current blockPos is a keypos for some zoom level
                        int matchedZoomLevel = 0;
                        for (int hzoom=MAX_ZOOM_USING_SOURCEFILE+1; hzoom<MAX_ZOOM_LEVELS; hzoom++)
                                {
                                if (blockPos % zoomStep[hzoom]==0)
                                        {
                                        matchedZoomLevel=hzoom;
                                        short max=lastRegionMax[matchedZoomLevel][ch];
                                        //short min=lastRegionMin[matchedZoomLevel][ch];
                                        short representativeSample;
                                        //if ( SOME CRITERIA - I DONT KNOW WHICH ONE YET )
                                                representativeSample = max;
                                        //else
                                        //      representativeSample = min;
                                        long long pl = posInLevel [ matchedZoomLevel ];
                                        RAMBuffer [ matchedZoomLevel ] [ pl ] = representativeSample;
                                        posInLevel [ matchedZoomLevel ]++;
                                        lastRegionMax[matchedZoomLevel][ch]=0;
                                        lastRegionMin[matchedZoomLevel][ch]=0;
                                        }
                                }
                        }
                blockPos++;
                }
        lcd->print("Rescanning Peaks : 100 % .." , 3);
        lcd->print("Rescanning Completed." , 4 );
        PMESG("Rescanning Peaks : Completed.");
        if (save()<0)
                {
                PMESG("WARNING : Cannot save Peak image (RAM Buffers) to a peak file");
                }
        PEXIT;
        return 1;
        }



int Peak::unbuild()
        {
	fclose(audioSourceFile);
        for (int i=0; i<MAX_ZOOM_LEVELS; i++)
                if (RAMBuffer[i])
                        delete RAMBuffer[i];
        return 1;
        }



int Peak::fill_buffer(short* bufferToFill, long long startBlock, int numBlocksToFill,  int hzoom)
        {
        int blocksFilled;
        if (hzoom > MAX_ZOOM_USING_SOURCEFILE)
                {
                long long offsetInBytes = (startBlock / zoomStep[hzoom]) * blockSize;
                long long offsetInShorts = offsetInBytes / sizeof(short) ;
                short* pi = RAMBuffer[hzoom] + offsetInShorts;
                short* pf = RAMBuffer[hzoom] + RAMBufferSize[hzoom];
                long long numShortsToFill = numBlocksToFill * channels;
                if (numShortsToFill >= (pf-pi))
                        numShortsToFill = pf-pi;
                if (numShortsToFill<0)
                        return 0;
                memcpy(bufferToFill, pi, numShortsToFill*sizeof(short));
                blocksFilled = numShortsToFill / channels;
                }
        else
                {
                long long stepInBytes = blockSize * zoomStep[hzoom];
                long long pos = startBlock * blockSize + headerSize;
                long long originalPos = ftell(audioSourceFile);
                blocksFilled = 0;
                short sho;
                short* p = bufferToFill;
                do
                        {
                        fseek(audioSourceFile, pos, SEEK_SET);
                        for (int ch=0; ch < channels; ch++)
                                {
                                fread(&sho, 1, sizeof(short), audioSourceFile);
                                *(p++)=sho;
                                }
                        blocksFilled++;
                        pos+=stepInBytes;
                        }
                while ((blocksFilled < numBlocksToFill) && (!feof(audioSourceFile)));
                fseek(audioSourceFile, originalPos, SEEK_SET);
                }
        return blocksFilled;
        }






void Peak::show_info()
        {
        PMESG("Peakfiles information\n");
        for (int i=0; i<MAX_ZOOM_LEVELS; i++)
                {
                //PMESG("Zoom 1:" << zoomStep[i] << "\t\t - "  << (i>MAX_ZOOM_USING_SOURCEFILE?"Using RAM buffer":"Using Source File") << "Peak step=" << zoomStep[i]);
                }
        }




int Peak::save()
        {
        PENTER;
        QString peakFilename = audioSource->rootDir + "/"+audioSource->filename + ".peak";
        PMESG("Saving Peak to %s", (const char*) peakFilename.latin1());
        size_t size ;
        FILE* file = fopen((const char*)peakFilename.latin1(),"w+");
        if (!file)
                {
                PERROR("Cannot save Peak image to %s", (const char*) peakFilename.latin1());
                PEXIT;
                return -1;
                }
        int totalsize = 0;
        for (int hzoom = MAX_ZOOM_USING_SOURCEFILE+1; hzoom<MAX_ZOOM_LEVELS; hzoom++)
                {
                totalsize += RAMBufferSize[hzoom];
                for (int j=0; j<RAMBufferSize[hzoom]; j++)
                        {
                        short sho = RAMBuffer[hzoom][j];
                        size = fwrite(&sho, 1 , sizeof(short), file);
                        }
                }
        PMESG("RAM Buffers saved. Total size of file is %d",totalsize * sizeof(short) );
        fclose(file);
        PEXIT;
        return 1;
        }


int Peak::load()
        {
        PENTER;
        QString peakFilename = audioSource->rootDir + "/"+ audioSource->filename + ".peak";
        PMESG("Looking for peak image file %s", (const char*) peakFilename.latin1());
        FILE* pfile = fopen((const char*)peakFilename.latin1(),"r+");
        if (!pfile)
                {
                PEXIT;
                return -1;
                }
        PMESG("Found it ! Loading it into RAM Buffers... ");
        for (int hzoom = MAX_ZOOM_USING_SOURCEFILE+1; hzoom<MAX_ZOOM_LEVELS; hzoom++)
                {
                for (int j=0; j<RAMBufferSize[hzoom]; j++)
                        {
                        short sho;
                        fread(&sho, 1 , sizeof(short), pfile);
                        if (feof(pfile)) sho=0;
                        RAMBuffer[hzoom][j]=sho;
                        }
                }
        PMESG("RAM Buffers sucessfully loaded.");
        fclose(pfile);
        PEXIT;
        return 1;
        }


void Peak::update_progress_bar() // DEPRECATED
        {
        PDEBUG("OLA!!");
        }
// eof
