/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include <string.h>
#include <mustux.h>
#include "PrafFile.hh"




#define BUFFER_SIZE 1024


PrafFile::PrafFile()
    {
    PENTERCONS;
    PEXITCONS;
    }


PrafFile::~PrafFile()
    {
    PENTERDES;
    PEXITDES;
    }


int PrafFile::open(char* newFileName)
    {
    PENTER;
    fileName=newFileName;
    file = fopen(fileName,"r");
    if (!file)
        {
        PEXIT;
        return -1;
        }
    PEXIT;
    return 1;
    }


int PrafFile::create(char* newFileName)
    {
    PENTER;
    fileName=newFileName;
    file = fopen(fileName,"w+");
    if (!file)
        {
        PEXIT;
        return -1;
        }
    PEXIT;
    return 1;
    }


int PrafFile::copy_to(char* filename)
        {
        FILE* newFile;
        newFile = fopen (filename,"w");
        if (newFile<0)
                {
                PERROR("Cannot copy PRAF file to %s", filename);
                return -1;
                }
        fwrite(&(formatLabel), 4, 1 , newFile);
        fwrite(&(channels), 4, 1 , newFile);
        fwrite(&(rate), 4, 1 , newFile);
        fwrite(&(bitDepth) , 4, 1 , newFile);
        fwrite(&(audioSize) , 8, 1 , newFile);
        fwrite(&(projectLabel), PRAF_HEADER_LABEL_SIZE, 1 , newFile);
        fwrite(&(groupLabel), PRAF_HEADER_LABEL_SIZE, 1 , newFile);
        fwrite(&(hardwareLabel), PRAF_HEADER_LABEL_SIZE, 1 , newFile);
        fwrite(&(description), PRAF_HEADER_DESCRIPTION_SIZE, 1 , newFile);
        fwrite(&(reserved), PRAF_HEADER_RESERVED_SIZE, 1, newFile);

        char buf[BUFFER_SIZE];
        size_t size;

        fseek(file, 0, SEEK_SET);
        while (!feof(file))
            {
            size = fread (&buf, 1, BUFFER_SIZE, file);
            fwrite(&buf, 1 , size, newFile);
            }
        fclose(newFile);
        return 1;
        }

int PrafFile::read_header()
    {
    PENTER;
    rewind();
    fread(&(formatLabel), 4, 1 , file);
    if ((formatLabel[0]!='P') ||
        (formatLabel[1]!='R') ||
        (formatLabel[2]!='A') ||
        (formatLabel[3]!='F'))
            {
            PERROR("This seems not to be a PRAF file.");
            PEXIT;
            return -1;
            }
    fread(&(channels), 4, 1 , file);
    fread(&(rate), 4, 1 , file);
    fread(&(bitDepth) , 4, 1 , file);
    fread(&(audioSize) , 8, 1 , file);
    fread(&(projectLabel), PRAF_HEADER_LABEL_SIZE, 1 , file);
    fread(&(groupLabel), PRAF_HEADER_LABEL_SIZE, 1 , file);
    fread(&(hardwareLabel), PRAF_HEADER_LABEL_SIZE, 1 , file);
    fread(&(description), PRAF_HEADER_DESCRIPTION_SIZE, 1 , file);
    fread(&(reserved), PRAF_HEADER_RESERVED_SIZE,1, file);
    headerSize=PRAF_HEADER_SIZE;
    PEXIT;
    return 1;
    }


static int lenght(char* b)
        {
        int x=0;
        while (b[x++]);
        return x-1;
        }


int PrafFile::write_header(int newChannels, int newRate, int newBitDepth, long long newAudioSize,
                             const char* newProjectLabel, const char* newGroupLabel, const char* newHardwareLabel,
                             const char* newDescription)
        {
        PENTER;
        formatLabel[0] = 'P';
        formatLabel[1] = 'R';
        formatLabel[2] = 'A';
        formatLabel[3] = 'F';
        int newProjectLabelLen = lenght((char*)newProjectLabel);
        int newGroupLabelLen = lenght((char*)newGroupLabel);
        int newHardwareLabelLen = lenght((char*)newHardwareLabel);
        int newDescriptionLen = lenght((char*)newDescription);
        for (int i=0; i<PRAF_HEADER_LABEL_SIZE; i++) projectLabel[i] = 0;
        for (int i=0; i<PRAF_HEADER_LABEL_SIZE; i++) groupLabel[i] = 0;
        for (int i=0; i<PRAF_HEADER_LABEL_SIZE; i++) hardwareLabel[i] = 0;
        for (int i=0; i<PRAF_HEADER_DESCRIPTION_SIZE; i++) description[i] = 0;
        for (int i=0; i<PRAF_HEADER_RESERVED_SIZE; i++) reserved[i]=0;
        for (int i=0; i < (int) newProjectLabelLen; i++) projectLabel[i] = newProjectLabel[i];
        for (int i=0; i < (int) newGroupLabelLen; i++) groupLabel[i] = newGroupLabel[i];
        for (int i=0; i < (int) newHardwareLabelLen; i++) hardwareLabel[i] = newHardwareLabel[i];
        for (int i=0; i < (int) newDescriptionLen; i++) description[i] = newDescription[i];
        for (int i=0; i<PRAF_HEADER_RESERVED_SIZE; i++) reserved[i] = 0;
        channels = newChannels;
        rate = newRate;
        bitDepth = newBitDepth;
        audioSize = newAudioSize;
        headerSize=PRAF_HEADER_SIZE;
        rewind();
        fwrite(&(formatLabel), 4, 1 , file);
        fwrite(&(channels), 4, 1 , file);
        fwrite(&(rate), 4, 1 , file);
        fwrite(&(bitDepth) , 4, 1 , file);
        fwrite(&(audioSize) , 8, 1 , file);
        fwrite(&(projectLabel), PRAF_HEADER_LABEL_SIZE, 1 , file);
        fwrite(&(groupLabel), PRAF_HEADER_LABEL_SIZE, 1 , file);
        fwrite(&(hardwareLabel), PRAF_HEADER_LABEL_SIZE, 1 , file);
        fwrite(&(description), PRAF_HEADER_DESCRIPTION_SIZE, 1 , file);
        fwrite(&(reserved), PRAF_HEADER_RESERVED_SIZE, 1, file);
        PEXIT;
        return 0;
        }



int PrafFile::close_file()
        {
        PENTER;
        int r = fclose(file);
        PEXIT;
        return r;
        }



int PrafFile::append_samples_from_wav(WavFile* wavFile)
        {
        PENTER;
        char buf[BUFFER_SIZE];
        size_t size;

        wavFile->rewind();
        wavFile->skip_header();
        skip_header();

        int i=0;
        FILE* input = wavFile->get_file();
        while (!feof(input))
            {
            size = fread (&buf, 1, BUFFER_SIZE, input);
            fwrite(&buf, 1 , size, file);
            i++;
            PMESG("read %d buffers", ++i );
            }
        PEXIT;
        return 1;
        }



int PrafFile::append_samples_from_buffer(char* buffer, int bufferSize)
        {
        size_t size ;
        size = fwrite(buffer, 1 , bufferSize, file);
        int r = 1;
        if ((int) size < bufferSize)
                {
                PERROR("Could not append the whole buffer to file");
                r = -1;
                }
        return r;
        }


void PrafFile::show_info()
        {
        PENTER;
        //PMESG("File       : " << fileName);
        //PMESG("Channels   : " << (channels == 1 ? "Mono" : "Stereo"));
        //PMESG("Rate       : " << rate << " Hz");
        //PMESG("bitDepth   : " << bitDepth << " bits");
        //PMESG("Project : " << projectLabel);
        //PMESG("Group : " + groupLabel);
        //PMESG("Hardware : " + hardwareLabel);
        //PMESG("Description : " + description);*/
        PEXIT;
        }

void PrafFile::rewind()
        {
        std::rewind(file);
        }


void PrafFile::skip_header()
        {
        fseek(file, headerSize, SEEK_SET);
        }


int PrafFile::get_channels()
        {
        return channels;
        }

int PrafFile::get_rate()
        {
        return rate;
        }

int PrafFile::get_bit_depth()
        {
        return bitDepth;
        }


long long PrafFile::get_audio_size()
        {
        return audioSize;
        }



long long PrafFile::get_file_size()
        {
	fseek(file, 0, SEEK_END);
        return ftell(file);
        }


int PrafFile::get_header_size()
        {
        return headerSize;
        }





char* PrafFile::get_project_label()
        {
        PENTER;
        char* cs = new char[PRAF_HEADER_LABEL_SIZE+1];
        strncpy(cs, projectLabel,PRAF_HEADER_LABEL_SIZE); cs[PRAF_HEADER_LABEL_SIZE]=0;
        PEXIT;
        return cs;
        }


char* PrafFile::get_group_label()
        {
        PENTER;
        char* cs = new char[PRAF_HEADER_LABEL_SIZE+1];
        strncpy(cs, groupLabel,PRAF_HEADER_LABEL_SIZE); cs[PRAF_HEADER_LABEL_SIZE]=0;
        PEXIT;
        return cs;
        }


char* PrafFile::get_hardware_label()
        {
        PENTER;
        char* cs = new char[PRAF_HEADER_LABEL_SIZE+1];
        strncpy(cs, hardwareLabel,PRAF_HEADER_LABEL_SIZE); cs[PRAF_HEADER_LABEL_SIZE]=0;
        PEXIT;
        return cs;
        }


char* PrafFile::get_description()
        {
        PENTER;
        char* cs = new char[PRAF_HEADER_DESCRIPTION_SIZE+1];
        strncpy(cs, description,PRAF_HEADER_DESCRIPTION_SIZE); cs[PRAF_HEADER_DESCRIPTION_SIZE]=0;
        PEXIT;
        return cs;
        }


FILE* PrafFile::get_file()
        {
        return file;
        }

char* PrafFile::get_filename()
        {
        return fileName;
        }

int PrafFile::fix_audio_size()
        {
        PENTER;
        fseek(file, 0, SEEK_END);
        long long fileSize = ftell(file);
        audioSize = fileSize - get_header_size();
        PMESG("Fixing audio size to %d", audioSize );
        fseek(file, 16, SEEK_SET);
        fwrite(&audioSize, 8, 1, file);
        PEXIT;
        return 1;
        }

// eof
