/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#ifndef PROJECTMANAGER_HH
#define PROJECTMANAGER_HH


#include <qlineedit.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qgroupbox.h>
#include <qslider.h>
#include <qprogressbar.h>
#include <qcheckbox.h>
#include <qdialog.h>
#include <qmessagebox.h>

class Project;
class FileBrowser;
class Interface;

class ProjectManager : public QDialog
        {
        Q_OBJECT

        public:
                ProjectManager( Interface* pParentInterface, const char* name = 0 );
                ~ProjectManager();

	        int create_new_project(QString projectName, int numSong);
	        int load_project(QString projectName);

	        bool project_is_current(QString title);
	        bool projectExists(QString title);
                /*
                const char* get_curr_project_name();
                int get_curr_project_num_songs();
                */


                Project* currentProject;

        public slots:
	        virtual void listAlbumProject_projectSelected();
                virtual void btnImportNow_clicked();
                virtual void btnSaveProject_clicked();
                virtual void btnLoadProject_clicked();
                virtual void btnCopyToProject_clicked();
                virtual void btnCreateProject_clicked();
                virtual void btnDeleteProject_clicked();
                virtual void btnSaveSong_clicked();
                virtual void btnSaveSongAs_clicked();
                virtual void btnSaveSongCopyAs_clicked();
                virtual void btnDeleteSource_clicked();
                virtual void btnRemoveAllClips_clicked();
                virtual void btnExportSource_clicked();
                virtual void btnSourcePlay_clicked();
                virtual void btnFilePlay_clicked();
                virtual void btnAddFileAsSource_clicked();
                virtual void btnAddSourceCDA_clicked();
                virtual void btnImportNowCDA_clicked();
                virtual void btnCDAPlay_clicked();
                virtual void btnRender_clicked();
                virtual void btnMaster_clicked();
                virtual void btnBurnCD_clicked();


                void show();

        private :
	        void set_current_project(Project* pProject);
                void update_projects_list();
	        int remove_recursively(QString pName);
	        int copy_recursively(QString pNameFrom, QString pNameTo);

                FileBrowser* fileBrowser;
	        Interface *parentInterface;


                QTabWidget* mainTabPane;
                QWidget* tab;
                QLabel* lblAlbumTitle;
                QLineEdit* fieldAlbumTitle;
                QLabel* lblNOfSongs;
                QLineEdit* fieldNOfSongs;
                QLabel* lblStatus;
                QComboBox* cmbStatus;
                QLabel* lblTotalDur;
                QLabel* lblEngineer;
                QLineEdit* fieldEngineer;

                QPushButton* btnRender;
                QPushButton* btnMaster;
                QPushButton* btnBurnCD;
                QPushButton* btnCreateProject;
                QPushButton* btnLoadProject;
                QPushButton* btnSaveProject;
                QPushButton* btnCopyToProject;
                QLineEdit* fieldSaveProjAs;
                QPushButton* btnDeleteProject;
                QListView* listAlbumProjects;
                QWidget* tab_2;
                QLabel* lblMasterGain;
                QLineEdit* fieldMasterGain;
                QLabel* lblNumTracks;
                QLineEdit* fieldNumTracks;
                QLabel* lblSongDur;
                QLineEdit* fieldSongDur;
                QLabel* lblArtist;
                QLineEdit* fieldArtists;
                QLabel* TextLabel2;
                QComboBox* cmbSongStatus;
                QPushButton* btnSaveSong;
                QPushButton* btnSaveSongAs;
                QLabel* lblOr;
                QPushButton* btn;
                QLineEdit* fieldCopyTo;
                QListView* listSongs;
                QWidget* tab_3;
                QPushButton* btnDeleteSource;
                QPushButton* btnRemoveAllClips;
                QPushButton* btnExportSource;
                QGroupBox* grpPreListenSource;
                QPushButton* btnSourcePlay;
                QLineEdit* fieldAudioSourceInfo;
                QLabel* lblSourceSpectro;
                QLabel* lblDSPVol1;
                QSlider* sliderSource;
                QCheckBox* checkSourceInfo;
                QCheckBox* checkSourceSpectro;
                QListView* listAudioSources;
                QWidget* tab_4;
                QLabel* lblFileBrowser_2;
                QGroupBox* grpPreListenFile;
                QLabel* lblDSPVol2;
                QSlider* sliderFile;
                QCheckBox* checkFileInfo;
                QCheckBox* checkFileSpectro;
                QLabel* lblFileSpectro;
                QLineEdit* fieldFileInfo;
                QPushButton* btnFilePlay;
                QGroupBox* grpImportFile;
                QLabel* lblFileDestSong;
                QLineEdit* fieldFileDestSong;
                QLabel* lblFileDestTrack;
                QLineEdit* fieldFileDestTrack;
                QLabel* lblFileDestPos;
                QLineEdit* fieldFileDestPos;
                QPushButton* btnAddFileAsSource;
                QPushButton* btnImportFileNow;
                QProgressBar* pbarFileImport;
                QCheckBox* checkFileFakeImport;
                QWidget* tab_5;
                QLabel* lblCDBrowser_2;
                QGroupBox* grpImportCDA;
                QLabel* lblDestSongCDA;
                QLineEdit* fieldDestSongCDA;
                QLabel* lblDestTrackCDA;
                QLineEdit* fieldDestTrackCDA;
                QLabel* lblDestPosCDA;
                QLineEdit* fieldDestPosCDA;
                QPushButton* btnAddSourceCDA;
                QPushButton* btnImportNowCDA;
                QProgressBar* pbarImportCDA;
                QCheckBox* checkFakeImportCDA;
                QGroupBox* grpPreListenCDA;
                QLabel* lblCDAVol;
                QSlider* sliderCDA;
                QPushButton* btnCDAPlay;
        };


#include "FileBrowser.hh"
#include "Project.hh"


#endif
