/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#ifndef RECORDER_HH
#define RECORDER_HH

#include <stdio.h>
#include <mustux.h>
#include <qthread.h>


class Audio;
class Song;
class Track;
class Recorder;

class ThRecLoop : public QThread
        {
        public:
                ThRecLoop(Recorder* pRecorder);
                void run();
        private:
                Recorder* recorder;
        };


class Recorder : public MustuxEngine
        {
        Q_OBJECT
        public:

                static const int REC_NONE = 0;
                static const int REC_ARMED = 1;
                static const int REC_RECORDING = 2;
                static const int REC_ABORTED = 3;
                static const int REC_FINISHED = 4;

                Recorder(Track* pTrack);
                ~Recorder();

                void set_output_file(QString fn);
                void set_capture_dir(QString cd);

                int arm();
                int rec();
                int force_stop();
                int stop();
                int disarm();

                int abort();
                int get_status();

                bool rec_frag();
                
                void set_bus_in(int bus);
                int get_bus_in();

        protected:
                void monitor_input();
                void draw_input();

                int destinationTrackNumber;
                Track* outputTrack ;
                long long initialInsertBlockPos;
                long long insertingBlockPos;
                int blockSize;

                QTimer* blinkTimer;
                Song* targetSong;
                int recStatus;
                bool blinkStatus;

                int channels;
                int rate;
                int bitDepth;
                QString outputFilename;
                QString captureDir;

                int recBufferSize;
                char* recBuffer;
                Audio* output;
                FILE* outputFile;
                float* inputLevel; // 0.0 ... 1.0
                short* max;

                ThRecLoop* thRecLoop;

                int busIn;

        public slots:
                void blink_rec_status();
        };

#include "Audio.hh"
#include "Track.hh"
#include "Song.hh"

#endif

//eof








