/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include <math.h>

#include <mustux.h>
#include "Slice.hh"

// This is for Gain : factor = db/6;


inline void mix_samples(short* destSample, short addingSample)
        {
        int newSample = *(destSample) + addingSample;
        newSample = newSample < MIN_SAMPLE ? MIN_SAMPLE : newSample;
        newSample = newSample > MAX_SAMPLE ? MAX_SAMPLE : newSample;
        *(destSample) = newSample;
        }

Slice::Slice( Mixer* pParentMixer, int anID )
        {
        ID = anID;
        parentMixer = pParentMixer;
        parentSong = parentMixer->get_parent_song();
        cellMixBuffer = (char*)0;
        numTracks = parentSong->get_total_tracks();
        subClipListPerTrack = new SubClipList[numTracks];
        for (int i=0; i < numTracks; i++)
                subClipListPerTrack[i]= (SubClipList) 0;

        }



Slice::~Slice()
        {
        for (int i=0; i < numTracks; i++)
                {
                SubClip* firstSubClip = subClipListPerTrack[i];
                SubClip* sc = firstSubClip;
                SubClip* sc2= firstSubClip;
                while (sc)
                        {
                        sc2=sc->next;
                        delete sc;
                        sc=sc2;
                        }
                }
        }


int Slice::add_sub_clip(SubClip* anSubClip, int trackNumber)
        {
        if (!subClipListPerTrack[trackNumber])
                {
                subClipListPerTrack[trackNumber]=anSubClip;
                subClipListPerTrack[trackNumber]->prev=0;
                subClipListPerTrack[trackNumber]->next=0;
                }
        else
                {
                SubClip* p = subClipListPerTrack[trackNumber];
                while (p->next) p=p->next;
                p->next=anSubClip;
                anSubClip->next=0;
                anSubClip->prev=p;
                }
        return 1;
        }




int Slice::mix(int bus, char* mixBuffer) // returns the last mixed byte
        {
        PENTER;
        // First detect if there is some "SOLO" track.
        bool thereAreSolos = false;
        for (int i=0; i<numTracks; i++)
                {
                if (parentSong->track[i]->is_solo())
                        {
                        thereAreSolos=true;
                        break;
                        }
                }

        int maxWidthInBytes = get_width_in_bytes();

        // allocate a temporary mix buffer, just for the track cell
        cellMixBuffer = new char[maxWidthInBytes];

        // clean the mix buffer
        memset(mixBuffer, 0, maxWidthInBytes);

        // this is the buffer for the source data, read from the file
        int maxMixBufferSamples = maxWidthInBytes / sizeof(short);
        int maxSourceBufferSamples=maxMixBufferSamples;
        int maxSourceBufferSize=maxSourceBufferSamples * sizeof(short);
        char* sourcebuffer = new char[maxSourceBufferSize];

        int maxPlayableBlockInSlice=0;
        int maxChannels=0;
        
        // begin the mixing process...
        bool clipping=false;
        for (int i=0; i<numTracks; i++)
                {
                // FIRST CHECK : do I have really to mix this track ?
                Track* thisTrack = parentSong->get_track(i);
                if (thisTrack->is_muted())
                        continue;
                if ((thereAreSolos) && (!thisTrack->is_solo()))
                        continue;
                if ((bus!=MustuxAudioDeviceMapper::ALL_BUSES) && (thisTrack->get_bus_out()!=bus))
                        continue;

                // Mixes all subclips in this cell (track vs slice cell)
                memset(cellMixBuffer, 0, maxWidthInBytes);
                SubClip* sc = subClipListPerTrack[i];

                while ((sc) && (!sc->parentClip->isMuted ))
                        {
                        // get the source file
                        Audio* audioSource = sc->audioSource;
                        FILE* file= audioSource->get_file();
                        audioSource->locate(sc->firstFileBlock);

                        // get the audio characteristics
                        int channels = audioSource->get_channels();
                        if (channels>maxChannels) maxChannels=channels;

                        int numSamplesToReadFromSource = sc->width * channels;
                        int numBytesToReadFromSource = numSamplesToReadFromSource * sizeof(short);

                        // read the audio from the file
                        int bytesRead = (int) fread(sourcebuffer, 1, numBytesToReadFromSource, file);
                        // bytesRead is not being used right now, but it can be in the future...
                        char* pSourceBuffer = sourcebuffer;

                        // calculate the destination point in the mixbuff, where the data will be mixed onto.
                        char* pCellMixBuffer = cellMixBuffer + (sc->relativeSliceBlock * channels * sizeof(short));

                        // Apply track Gain and Pan
                        float trackGain = sc->parentClip->parentTrack->assocTrackPanel->get_gain();
                        float gainToApply = (trackGain>=0.0f)? 1.0f + trackGain/6.0f : 1.0f / pow(2.0f,-1.0f*trackGain/6.0f);
                        float panFactor = sc->parentClip->parentTrack->assocTrackPanel->get_pan();
                        float panL=1.0;
                        float panR=1.0;
                        panR = panFactor>=0 ? 1.0 : 1.0 + panFactor;
                        panL = panFactor>=0 ? 1.0 - panFactor : 1.0;
                        int sampleWithAppliedGain;
                        int srcCounter=0;
                        int channel=0;

                        while (srcCounter < numSamplesToReadFromSource ) // && (playable))
                                {
                                short* sps = (short*) pSourceBuffer;
                                short sample = *(sps);
                                float fSampleWithAppliedGain = (float) sample * gainToApply;

                                // apply track PAN int the sample (only if it is a stereo clip)
                                fSampleWithAppliedGain *= channel==1 ? panR : panL;
                                sampleWithAppliedGain = (short) fSampleWithAppliedGain;
                                short* spm = (short*) pCellMixBuffer;
                                mix_samples(spm,sampleWithAppliedGain);


                                // make a copy of sample in case it is mono , so it becomes "stereo" ;-)
                                if (channels==1)
                                        {
                                        pCellMixBuffer+=sizeof(short);
                                        spm = (short*) pCellMixBuffer;
                                        mix_samples(spm,sampleWithAppliedGain);
                                        }

                                pSourceBuffer+=sizeof(short);
                                pCellMixBuffer+=sizeof(short);
                                srcCounter++;
                                if (++channel>1) channel=0;
                                }
                        int lastSubClipBlock =sc->relativeSliceBlock+sc->width;
                        if (lastSubClipBlock>maxPlayableBlockInSlice)
                                maxPlayableBlockInSlice=lastSubClipBlock;
                        sc=sc->next;
                        }

                // Process Track Filters
                // FOR LATER :  if (thisTrack->filterChain->is_active())

                thisTrack->filterChain->process(cellMixBuffer,maxWidthInBytes);

                // Mix the mixed cell into the slice mix buffer
                char* pMixBuffer = mixBuffer;
                char* pCellMixBuffer = cellMixBuffer;
                int c=0;

                while (c<maxWidthInBytes)
                        {
                        mix_samples((short*)pMixBuffer,*((short*)pCellMixBuffer));
                        pMixBuffer+=sizeof(short);
                        pCellMixBuffer+=sizeof(short);
                        c+=sizeof(short);
                        }
                }

        delete cellMixBuffer;
        delete sourcebuffer;
        if (clipping)
                {
                // do something ...
                }
        PEXIT;
        return maxPlayableBlockInSlice * sizeof(short) * maxChannels;
        }




int Slice::render(char* renderBuffer)
        {
        int nbytesmixed = mix(MustuxAudioDeviceMapper::ALL_BUSES, renderBuffer);
        return nbytesmixed;
        }



void Slice::show()
        {
        //PDEBUG("Subclip schema for Slice " << ID );
        for (int i=0; i<numTracks; i++)
                {
                //PDEBUG("        TRACK " << i);
                SubClip* sc = subClipListPerTrack[i];
                while (sc)
                        {
                        //PDEBUG("               SubClip  " << sc <<" : RSP =" << sc->relativeSliceBlock <<" , Width = " << sc->width << " , FSB =" << sc->firstFileBlock);
                        sc=sc->next;
                        }
                }
        }


int Slice::get_width() // substituir por uma variavel simples, definida e incializada no constructor
        {
        return rightBorder - leftBorder;
        }


int Slice::get_width_in_bytes() // substituir por uma variavel simples, definida e incializada no constructor
        {
        int mc = parentMixer->get_max_channels();
        int x = (rightBorder - leftBorder)*mc*sizeof(short);
        return x;
        }

//eof

