/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/

#include <mustux.h>
#include "Track.hh"
#include "ColorManager.hh"


Track::Track(Song* pParentSong, int pBaseY, int pHeight) : MustuxDrawableRegion( pParentSong->get_tracks_area() )
        {
        PENTERCONS;
        parentSong = pParentSong;
        baseY = pBaseY;
        height = pHeight;
        clipList = (AudioClip*) 0;
        filterChain = new FilterChain(this);
        recorder = new Recorder(this);
        isActive = false;
        isMuted = false;
        isSolo = false;
        busIn = 0;
        busOut = 0;
        PEXITCONS;
        }


Track::~Track()
        {
        PENTERDES;
        delete filterChain;
        PEXITDES;
        }

void Track::set_assoc_track_panel(TrackPanel* pAssocTrackPanel)
        {
        assocTrackPanel = pAssocTrackPanel;
        }

void Track::recreate()
        {
        if (isActive)
                {
                deactivate();
                clear();
                draw();
                update();
                activate();
                }
        else
                {
                clear();
                draw();
                update();
                }
        }


void Track::update()
        {
        drawArea->update(0,baseY,drawArea->width(),height);
        }


void Track::clear()
        {
        drawArea = parentSong->get_tracks_area();
        QPainter* p = drawArea->painter;
        p->fillRect(0, baseY, drawArea->width() , height, ColorManager::get(ColorManager::COLOR_TRACK_BG));
        p->setPen(QColor(100,100,100));// This is intentionally fixed color
        p->drawLine(0, baseY + height, drawArea->width(), baseY + height);
        p->setPen(QColor(240,240,240));// This is intentionally fixed color.
        p->drawLine(0, baseY + height -1 , drawArea->width(), baseY + height -1);
        }



void Track::toggle_active_rect()
        {
        drawArea = parentSong->get_tracks_area();
        drawArea->painter->setRasterOp(Qt::XorROP);
        //if (parentSong->editingMode==Song::EDITING_MODE_TRACK_CURVES)
                //drawArea->painter->setPen(QColor(255,255,255));
        //else
        drawArea->painter->setPen(ColorManager::get(ColorManager::COLOR_TRACK_ACTIVE_BORDER));
        for (int i=0; i<5; i++)
                drawArea->painter->drawRect( i , baseY + i + 1 , drawArea->width() - i*2 , height - i*2 -1 );
        drawArea->painter->setRasterOp(Qt::CopyROP);
        drawArea->update();
        }


int Track::draw()
        {
        AudioClip* c=clipList;
        while (c)
                {
                c->draw();
                c=c->next;
                }
        if (parentSong->editingMode==Song::EDITING_MODE_TRACK_CURVES)
                filterChain->draw();

        /*// just a test
        QPixmap pixmap;
        if (!pixmap.load("images/button_bg00.jpg"))
                {
                PERROR("Cannot load image");
                }
        drawArea->painter->drawPixmap(1,baseY+1, pixmap );
        drawArea->update();
        */
        return 1;
        }



bool Track::is_pointed()
        {
        drawArea = parentSong->tracksArea;
        int x = drawArea->mapFromGlobal(QCursor::pos()).x();
        int y = drawArea->mapFromGlobal(QCursor::pos()).y();
        bool b = ( (x >= 0)  &&
                   (x <= drawArea->width() ) &&
                   (y >= baseY ) &&
                   (y <= baseY + height )
                 );
        return b ;
        }



int Track::split_clip(long long splitPoint)
        {
        PMESG("Trying to find clip under %d",splitPoint);
        AudioClip* c = get_clip_under(splitPoint);
        if (c!=0)
                {
                long long leftLenght = splitPoint - (c->trackFirstBlock);
                long long clipLenght = c->sourceLastBlock - c->sourceFirstBlock;
                long long rightLenght = clipLenght - leftLenght;
                long long rightSourceFirstBlock = c->sourceFirstBlock + leftLenght;
                c->sourceLastBlock = c->sourceFirstBlock + leftLenght;
                c->trackLastBlock = c->trackFirstBlock + leftLenght;
                add_clip(c->get_audio_source(), splitPoint, rightSourceFirstBlock, rightLenght, c->isTake);
                }
        else
                {
                PMESG("I am upon no clip :-(");
                }
        recreate();
        return 0;
        }




AudioClip* Track::get_clip_under(long long blockPos)
        {
        AudioClip* theClip = (AudioClip*) 0;
        AudioClip* c=clipList;
        while (c)
                {
                long long fb = c->trackFirstBlock;
                long long lb = c->trackLastBlock;
                if ((fb<=blockPos) && (lb>=blockPos))
                        {
                        theClip = c;
                        }
                if (fb>blockPos) break;
                c=c->next;
                }
        return theClip;
        }



AudioClip* Track::get_next_clip_after(long long blockPos)
        {
        AudioClip* c=clipList;
        while (c)
                {
                if (c->trackFirstBlock > blockPos)
                        {
                        return c;
                        }
                c=c->next;
                }
        return (AudioClip*) 0;
        }



int Track::delete_clip(AudioClip* clip)
        {
        PENTER;
        if (!clip)
                {
                PERROR("Trying to delete invalid Clip");
                PEXIT;
                return -1;
                }
        if (!clip->prev) // if it is the first clip...
                {
                clipList=clip->next;
                if (clipList)
                        clipList->prev=0;
                delete clip;
                PEXIT;
                return 1;
                }
        if (clip->next==0) // if it is the last clip...
                {
                clip->prev->next=0;
                delete clip;
                PEXIT;
                return 1;
                }
        clip->prev->next = clip->next;
        clip->next->prev = clip->prev;
        delete clip;

        // just for debug
        /*PMESG("Entire clip list for track " << assocTrackPanel->ID << " now is ");
        AudioClip* c=clipList;
        int i=0;
        while (c)
                {
                PMESG("Clip #" << i++
                << " : SFB=" << c->sourceFirstBlock
                << " , SLB=" <<c->sourceLastBlock
                << " , TRB=" <<c->trackFirstBlock);
                c=c->next;
                }
        */
        
        PEXIT;
        return 1;
        }



AudioClip* Track::add_clip(Audio* audioSource, long long trackInsertBlock, bool isTake)
        {
        PENTER;
        PMESG("Creating clip starting in %d" ,trackInsertBlock);
        AudioClip* c = add_clip(audioSource, trackInsertBlock, 0, audioSource->get_total_blocks(),isTake);
        recreate();
        PEXIT;
        return c;
        }



AudioClip* Track::add_clip(Audio* audioSource, long long trackInsertPos, long long sourceFirstBlock,
                           long long lenght, bool isTake)
        {
        PENTER;
        AudioClip *clip = new AudioClip(this,
                                        audioSource,
                                        sourceFirstBlock,
                                        lenght,
                                        trackInsertPos);
        clip->isTake = isTake;
        AudioClip* c;
        AudioClip* c2;

        if (!clipList)
                {
                long long clipLenght = clip->sourceLastBlock - clip->sourceFirstBlock;
                /*PMESG("Adding first clip :  [ TFB = " << clip->trackFirstBlock <<
                                        " , TLB = " << clip->trackLastBlock <<
                                        " , SFB = " << clip->sourceFirstBlock <<
                                        " , SLB = " << clip->sourceLastBlock <<
                                        " , LEN = " << clipLenght << " ]");
                */
                clipList = clip;
                clip->prev=0;
                clip->next=0;
                }
        else
                {
                c=clipList;
                c2=clipList;
                while (c &&  (c->trackFirstBlock < trackInsertPos))
                        {
                        c2=c;
                        c=c->next;
                        }
                if (!c)
                        {
                        PMESG("Adding at the end of list");
                        c2->next=clip;
                        clip->prev=c2;
                        clip->next=0;
                        }
                else 
                        {
                        if (c==clipList)
                                {
                                PMESG("Adding at the begin of list");
                                clip->next=clipList;
                                clipList->prev=clip;
                                clip->prev=0;
                                clipList=clip;
                                }
                        else
                                {
                                PMESG("Adding in the middle of list");
                                clip->next=c2->next;
                                clip->prev=c2;
                                c2->next->prev=clip;
                                c2->next=clip;
                                }
                        }
                }
                
        // just for debug
        /*
        PMESG("Entire clip list for track " << assocTrackPanel->ID << " now is ");
        c=clipList;
        int i=0;
        while (c)
                {
                PMESG("Clip #" << i++
                << " : SFB=" << c->sourceFirstBlock
                << " , SLB=" <<c->sourceLastBlock
                << " , TRB=" <<c->trackFirstBlock);
                c=c->next;
                }
        */
        PEXIT;
        return clip;
        }

void Track::activate()
        {
        if (!isActive)
                {
                isActive=true;
                toggle_active_rect();
                int cfc  = filterChain->currentFilterController;
                if (cfc>=0)
                        filterChain->filterController[cfc]->start_blinking();
                }
        }


void Track::deactivate()
        {
        if (isActive)
                {
                int cfc  = filterChain->currentFilterController;
                if (cfc>=0)
                        filterChain->filterController[cfc]->deactivate();
                toggle_active_rect();
                isActive=false;
                }
        }




void Track::set_blur(bool stat)
        {
        ColorManager::set_blur(ColorManager::COLOR_TRACK_BG , stat);
        AudioClip* c=clipList;
        while (c)
                {
                c->set_blur(stat);
                c=c->next;
                }

        }


int Track::get_baseX() // the implementation for track is simply return 0
        {
        return 0;
        }

int Track::get_baseY()
        {
        return baseY;
        }

int Track::get_width() // the implementation for tracks is simply return the drawArea widths
        {
        return drawArea->width();
        }


int Track::get_height()
        {
        return height;
        }


QString Track::get_name()
        {
        return assocTrackPanel->name;
        }

int Track::toggle_recordable()
        {
        PENTER;
        if (recorder->get_status()==Recorder::REC_NONE)
                {
                int r=-1;
                QString captureDir = parentSong->captureDir;
                recorder->set_capture_dir(captureDir);
                QDate d = QDate::currentDate();
                QTime t = QTime::currentTime();
                QString s1;s1.setNum(d.year());
                QString s2;s2.setNum(d.month());
                QString s3;s3.setNum(d.day());
                QString s4;s4.setNum(t.hour());
                QString s5;s5.setNum(t.minute());
                QString s6;s6.setNum(t.second());
                QString tns = "take-"+s1+s2+s3+"-"+s4+s5+s6+".praf";
                recorder->set_output_file(tns);
                recorder->set_bus_in(busIn);
                r = recorder->arm();
                if (r>0)
                        assocTrackPanel->start_vu();
                recreate();
                }
        else if (recorder->get_status()==Recorder::REC_ARMED)
                {
                recorder->disarm();
                assocTrackPanel->stop_vu();
                recreate();
                }
        else
                {
                PEXIT;
                return -1;
                }
        PEXIT;
        return 1;
        }


int Track::toggle_record()
        {
        PENTER;
        if (recorder->get_status()==Recorder::REC_ARMED)
                {
                recorder->rec();
                }
        else if (recorder->get_status()==Recorder::REC_RECORDING)
                {
                recorder->force_stop();
                }
        else
                {
                PEXIT;
                return -1;
                }
        PEXIT;
        return 1;
        }



int Track::get_recorder_status()
        {
        return recorder->get_status();
        }


int Track::stop()
        {
        filterChain->cleanup();
        }


Song* Track::get_parent_song()
        {
        return parentSong;
        }


int Track::get_bus_in()
        {
        return busIn;
        }

int Track::get_bus_out()
        {
        return busOut;
        }


void Track::set_bus_in(int b)
        {
        busIn=b;
        }

void Track::set_bus_out(int b)
        {
        busOut=b;
        }


bool Track::is_muted()
        {
        return isMuted;
        }

bool Track::is_solo()
        {
        return isSolo;
        }


bool Track::is_recordable()
        {
        return (recorder->get_status()==Recorder::REC_ARMED);
        }

bool Track::is_active()
        {
        return isActive;
        }

void Track::toggle_active()
        {
        isActive=!isActive;
        }

QString Track::get_schema()
        {
        PENTER;
        QString sint;
        QString output="";
        AudioClip* cli;
        sint.setNum(assocTrackPanel->ID);
        output = output + "<track id=\"" + sint + "\"";
        output = output + " name=\"" + assocTrackPanel->name + "\"";
        output = output + " gain=0.0 pan=0.0 mute=0 solo=0 busin=0 busout=0";
        output = output +  ">\n";
        cli = clipList;
        QString tag;
        while (cli)
                {
                if (cli->isTake)
                        tag="take";
                else
                        tag="source";
                output = output +  "       <clip " + tag + "=\"" + cli->audioSource->get_filename() + "\"";

                sint.setNum((double)cli->trackFirstBlock);
                output = output +  " TFB=\"" + sint;
                sint.setNum((double)cli->sourceFirstBlock);
                output = output +  "\" SFB=\"" + sint;
                sint.setNum((double)cli->get_lenght());
                output = output +  "\" LEN=\"" + sint + "\">\n";
                cli=cli->next;
                }
        output = output +  filterChain->get_schema();
        output = output +  "</track>\n";
        PEXIT;
        return output;
        }
// eof

