/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include <qcolor.h>
#include <mustux.h>
#include "TrackPanel.hh"
#include "ColorManager.hh"






TrackPanel::TrackPanel(Track* pAssocTrack, int pID, QString pName, int pBaseY, int pHeight)
        : MustuxDrawableRegion( pAssocTrack->get_parent_song()->get_tracks_area() )
        {
        PENTERCONS;
        ID=pID;
        assocTrack = pAssocTrack;
        drawArea = assocTrack->get_parent_song()->get_track_panels_area();
        baseY = pBaseY;
        height = pHeight;
        name=pName;
        gain = 0.0;
        pan = 0.0;
        vu = new TrackPanelVU(this);
        PEXITCONS;
        }


TrackPanel::~TrackPanel()
        {
        PENTERDES;
        delete vu;
        PEXITDES;
        }


void TrackPanel::recreate()
        {
        if (assocTrack->is_active())
                {
                deactivate();
                clear();
                draw();
                update();
                activate();
                }
        else
                {
                clear();
                draw();
                update();
                }
        }


void TrackPanel::update()
        {
        drawArea->update(0,baseY,drawArea->width(),height);
        }



void TrackPanel::clear()
        {
        drawArea->painter->fillRect(0, baseY, drawArea->width() , height, ColorManager::get(ColorManager::COLOR_TRACK_PANEL_BG_COLOR));
        }



void TrackPanel::toggle_active_rect()
        {
        drawArea->painter->setRasterOp(Qt::XorROP);
        drawArea->painter->setPen(ColorManager::get(ColorManager::COLOR_TRACK_ACTIVE_BORDER));
        for (int i=0; i<3; i++)
                drawArea->painter->drawRect( i + 2  , baseY + i + 1 , drawArea->width() - i*2 -2 , height - i*2 - 1 );
        drawArea->painter->setRasterOp(Qt::CopyROP);
        drawArea->update();
        }


void TrackPanel::draw()
        {
        QPainter* p = drawArea->painter;
        p->setPen(QColor(20,25,55));
        p->setFont( QFont( "Helvetica", 6) );
        QString sid; sid.setNum(ID+1);
        sid = sid + " - " + name;
        p->fillRect(0,baseY,drawArea->width(),20,QColor(170,180,190));
        p->drawText(10,baseY+15, sid);


        int sliderWidth=drawArea->width()-45;
        int sliderx=10;
        QString s,sgain,sv;
        int y;
        float v;

        y = baseY+30;
        p->drawText(sliderx, y, "GAIN");
        sgain.setNum(gain,'f',1);
        s = ( gain > 0 ? QString("+") + sgain :  sgain );
        v = gain/25;
        p->fillRect(sliderx+30,y-8,sliderWidth,10,QColor(200,200,240));
        p->drawRect(sliderx+30,y-8,sliderWidth,10);
        p->drawRect(sliderx+30+sliderWidth/2+(int)(v*sliderWidth/2)-1,y-8,1,10);
        p->drawText(sliderx+30+sliderWidth/2-8,y, s);

        y=baseY+45;
        p->drawText(sliderx, y, "PAN");
        v = pan;
        sv.setNum(v,'f',1);
        s = ( v > 0 ? QString("+") + sv :  sv );
        p->fillRect(sliderx+30,y-8,sliderWidth,10,QColor(200,200,240));
        p->drawRect(sliderx+30,y-8,sliderWidth,10);
        p->drawRect(sliderx+30+sliderWidth/2+(int)(v*sliderWidth/2)-1,y-8,1,10);
        p->drawText(sliderx+30+sliderWidth/2-8,y, s);

        const int MUTE_BUTTON_X = 90;
        const int MUTE_BUTTON_Y = baseY+4;
        const int MUTE_BUTTON_W = 19;
        const int MUTE_BUTTON_H = 15;

        const int SOLO_BUTTON_X = 115;
        const int SOLO_BUTTON_Y = baseY+4;
        const int SOLO_BUTTON_W = 19;
        const int SOLO_BUTTON_H = 15;

        const int REC_BUTTON_X = 140;
        const int REC_BUTTON_Y = baseY+4;
        const int REC_BUTTON_W = 19;
        const int REC_BUTTON_H = 15;

        p->setFont( QFont( "Helvetica", 6) );
        if (assocTrack->is_muted())
                {
                p->setPen(ColorManager::get(ColorManager::COLOR_TRACK_PANEL_FG_COLOR));
                p->fillRect(MUTE_BUTTON_X,MUTE_BUTTON_Y,MUTE_BUTTON_W,MUTE_BUTTON_H, ColorManager::get(ColorManager::COLOR_TRACK_PANEL_GREEN));
                p->drawRect(MUTE_BUTTON_X,MUTE_BUTTON_Y,MUTE_BUTTON_W,MUTE_BUTTON_H);
                p->drawText(MUTE_BUTTON_X + 2, MUTE_BUTTON_Y + 10, "MU");
                }
        else
                {
                p->setPen(ColorManager::get(ColorManager::COLOR_TRACK_PANEL_FG_COLOR));
                p->fillRect(MUTE_BUTTON_X, MUTE_BUTTON_Y, MUTE_BUTTON_W, MUTE_BUTTON_H, ColorManager::get(ColorManager::COLOR_TRACK_PANEL_BG_COLOR));
                p->drawRect(MUTE_BUTTON_X, MUTE_BUTTON_Y, MUTE_BUTTON_W, MUTE_BUTTON_H);
                p->drawText(MUTE_BUTTON_X + 2, MUTE_BUTTON_Y + 10, "MU");
                }

        if (assocTrack->is_solo())
                {
                p->setPen(ColorManager::get(ColorManager::COLOR_TRACK_PANEL_FG_COLOR));
                p->fillRect(SOLO_BUTTON_X,SOLO_BUTTON_Y,SOLO_BUTTON_W,SOLO_BUTTON_H, ColorManager::get(ColorManager::COLOR_TRACK_PANEL_GREEN));
                p->drawRect(SOLO_BUTTON_X,SOLO_BUTTON_Y,SOLO_BUTTON_W,SOLO_BUTTON_H);
                p->drawText(SOLO_BUTTON_X + 2, SOLO_BUTTON_Y + 10, "SO");
                }
        else
                {
                p->setPen(ColorManager::get(ColorManager::COLOR_TRACK_PANEL_FG_COLOR));
                p->fillRect(SOLO_BUTTON_X, SOLO_BUTTON_Y, SOLO_BUTTON_W, SOLO_BUTTON_H, ColorManager::get(ColorManager::COLOR_TRACK_PANEL_BG_COLOR));
                p->drawRect(SOLO_BUTTON_X, SOLO_BUTTON_Y, SOLO_BUTTON_W, SOLO_BUTTON_H);
                p->drawText(SOLO_BUTTON_X + 2, SOLO_BUTTON_Y + 10, "SO");
                }

        if (assocTrack->is_recordable())
                {
                p->setPen(ColorManager::get(ColorManager::COLOR_TRACK_PANEL_FG_COLOR));
                p->fillRect(REC_BUTTON_X,REC_BUTTON_Y,REC_BUTTON_W,REC_BUTTON_H, ColorManager::get(ColorManager::COLOR_TRACK_PANEL_RED));
                p->drawRect(REC_BUTTON_X,REC_BUTTON_Y,REC_BUTTON_W,REC_BUTTON_H);
                p->drawText(REC_BUTTON_X + 2, REC_BUTTON_Y + 10, "RE");
                }
        else
                {
                p->setPen(ColorManager::get(ColorManager::COLOR_TRACK_PANEL_FG_COLOR));
                p->fillRect(REC_BUTTON_X, REC_BUTTON_Y, REC_BUTTON_W, REC_BUTTON_H, ColorManager::get(ColorManager::COLOR_TRACK_PANEL_BG_COLOR));
                p->drawRect(REC_BUTTON_X, REC_BUTTON_Y, REC_BUTTON_W, REC_BUTTON_H);
                p->drawText(REC_BUTTON_X + 2, REC_BUTTON_Y + 10, "RE");
                }
        int inRoute = assocTrack->get_bus_in();
        int outRoute = assocTrack->get_bus_out();
        p->setFont( QFont( "Helvetica", 8) );
        QString sir; sir.setNum(inRoute);
        QString sor; sor.setNum(outRoute);
        QString sr = "IN:"+sir + "   OUT:" + sor;
        p->drawText(10, baseY + 60, sr);

        }




bool TrackPanel::is_pointed()
        {
        int x = drawArea->mapFromGlobal(QCursor::pos()).x();
        int y = drawArea->mapFromGlobal(QCursor::pos()).y();
        bool b = ((x>=0) && (x<=0+drawArea->width()) && (y>=baseY) && (y<=baseY+height));
        return b ;
        }



void TrackPanel::activate()
        {
        toggle_active_rect();
        }


void TrackPanel::deactivate()
        {
        toggle_active_rect();
        }


void TrackPanel::set_gain(float pGain)
        {
        if ( pGain < -24.0 )
                gain=-24.0;
        else
        if ( pGain > 24.0 )
                gain=24.0;
        else
                gain=pGain;
        }

void TrackPanel::set_pan(float pPan)
        {
        if ( pPan < -1.0 )
                pan=-1.0;
        else
        if ( pPan > 1.0 )
                pan=1.0;
        else
                pan=pPan;
        }

void TrackPanel::start_vu()
        {
        vu->activate();
        recreate();
        }

void TrackPanel::stop_vu()
        {
        vu->deactivate();
        recreate();
        }

TrackPanelVU* TrackPanel::get_vu()
        {
        return vu;
        }

// eof

