/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/

#ifndef TRACKPANEL_HH
#define TRACKPANEL_HH

#include <mustux.h>

/** TrackPanel is not a Widget. It is object that DRAWS on a region of widget to
  * display the track main information, such gain, volume, pan, channel, and so on...
  * the drawArea atribute must be the widget inside the associated Song that will be used
  * to display the contents of the track panel
  */

class Track;
class Mixer;
class TrackPanelVU;

class TrackPanel : public MustuxDrawableRegion
        {

        public :
                TrackPanel(Track* pAssocTrack,  int pID, QString pName, int pBaseY, int pHeight);
                ~TrackPanel();

                void recreate();
                void activate();
                void deactivate();
                void draw_axis();
                void update();
                void clear();
                void draw();
                void set_gain(float pGain);
                void set_pan(float pPan);
                bool is_pointed();
                int get_ID() { return ID; };
                float get_gain() { return gain; };
                float get_pan() { return pan; };
                int get_base_Y() { return baseY; };
                QString get_name() { return name; };
                void toggle_active_rect();
                void start_vu();
                void stop_vu();
                TrackPanelVU* get_vu();
                bool is_solo();
                bool is_muted();
                bool is_active();
                bool is_recordable();

        private :
                Track* assocTrack;
                TrackPanelVU* vu;

                int ID;
                int baseY;
                int height;
                QString name;
                float gain;
                float pan;
                friend class Song;
	        friend class Mixer;
                friend class Slice;
                friend class TrackPanelVU;
                friend class Track;
                friend class Recorder;

        };


#include "Track.hh"
#include "Mixer.hh"
#include "TrackPanelVU.hh"

#endif



