/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include <qcolor.h>

#include <mustux.h>

#include "TrackPanelVU.hh"

static const int MAX_LEVEL = 100;
static const int ROOT_X = 25;

// This is for VU : db = 20 * log ( sample / MaxSample )

TrackPanelVU::TrackPanelVU(TrackPanel* pAssocTrackPanel, int pChannels , int pRate , int pBitDepth)
        {
        PENTERCONS;
        channels = pChannels;
        rate = pRate;
        bitDepth = pBitDepth;

        assocTrackPanel=pAssocTrackPanel;
        drawArea = assocTrackPanel->drawArea;
        isActive=false;
        lCurrLevel = 0;
        rCurrLevel = 0;
        //dsp =  new MustuxDsp();

        updateTimer = new QTimer(this);
        connect( updateTimer , SIGNAL(timeout()), this, SLOT(update()));

        PEXITCONS;
        }


TrackPanelVU::~TrackPanelVU()
        {
        PENTERDES;
        delete updateTimer;
        PEXITDES;
        }


void TrackPanelVU::update_levels(float* levels) // SHOULD BE SYNCHRONIZED
        {
        // the lCurrLevel and rCurrLevel variables should become an array
        // to be coherent with the short* levels parameters.
        // for now, it just uses the [0] and [1] elements. VERY DANGEROUS
        lCurrLevel = (int) (levels[0] * MAX_LEVEL);// (lCurrLevel>levels[0]) ? lCurrLevel-5 : levels[0];
        rCurrLevel = (int) (levels[1] * MAX_LEVEL);// (rCurrLevel>levels[1]) ? rCurrLevel-5 : levels[1];
        }



void TrackPanelVU::update()
        {
        int baseY = assocTrackPanel->baseY+30;

	// First clear where is not used
        drawArea->painter->setPen(QColor(20,20,20));
        for (int i=lCurrLevel; i<MAX_LEVEL; i++)
                drawArea->painter->drawLine(25 + i, baseY+32, 25 + i, baseY+40);
        for (int i=rCurrLevel; i<MAX_LEVEL; i++)
                {
                drawArea->painter->drawLine(25 + i, baseY+42, 25 + i, baseY+50);
                }

        if (lCurrLevel>=MAX_LEVEL)
                {
                drawArea->painter->fillRect( ROOT_X + MAX_LEVEL + 5, baseY+32, 26,9,QColor(250,0,0));
                drawArea->painter->setPen(QColor(0,0,0));
                drawArea->painter->setFont( QFont( "Helvetica", 6) );
                drawArea->painter->drawText( ROOT_X + MAX_LEVEL + 10, baseY+39, "OVR");
                }
        if (rCurrLevel>=MAX_LEVEL)
                {
                drawArea->painter->fillRect( ROOT_X + MAX_LEVEL + 5, baseY+42, 26, 9, QColor(250,0,0) );
                drawArea->painter->setPen( QColor(0,0,0) );
                drawArea->painter->setFont( QFont( "Helvetica", 6) );
                drawArea->painter->drawText( ROOT_X + MAX_LEVEL + 10, baseY+49, "OVR");
                }


        // draw the peak meter
        for (int i = 0; i < lCurrLevel; i++)
                {
                drawArea->painter->setPen(QColor( i*2,255-(i>(MAX_LEVEL/2)?(i-(MAX_LEVEL/2))*3:0),0) );
                drawArea->painter->drawLine( ROOT_X + i, baseY+32, 25 + i, baseY+40);

                }
        for (int i = 0; i < rCurrLevel; i++)
                {
                drawArea->painter->setPen(QColor( i*2,255-(i>(MAX_LEVEL/2)?(i-(MAX_LEVEL/2))*3:0),0) );
                drawArea->painter->drawLine( ROOT_X + i,baseY+42, 25 + i, baseY+50 );
                }
        drawArea->update(ROOT_X,baseY+32, MAX_LEVEL + 50,20);
        }


void TrackPanelVU::clear()
        {
        int baseY = assocTrackPanel->baseY;
        drawArea->painter->setPen(QColor(20,20,20));
        drawArea->painter->setFont( QFont( "Helvetica", 8) );
        drawArea->painter->drawText(10, baseY+40, "L");
        drawArea->painter->drawText(10, baseY+50, "R");
        for (int i=0; i<MAX_LEVEL; i++)
                {
                drawArea->painter->drawLine(25 + i, baseY+32, 25 + i, baseY + 40);
                drawArea->painter->drawLine(25 + i, baseY+42, 25 + i, baseY + 50);
                }
        drawArea->update(25,baseY+32, 150, 50);
        }




int TrackPanelVU::activate()
        {
        isActive=true;
        clear();
        updateTimer->start(40);
        return 1;
        }


int TrackPanelVU::deactivate()
        {
        int r=1;
        isActive=false;
        updateTimer->stop();
        return r;
        }


bool TrackPanelVU::is_active()
        {
        return isActive;
        }

// eof

