/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include <string.h>
#include "WavFile.hh"
#include <mustux.h>

WavFile::WavFile()
        {
        PENTERCONS;
        PEXITCONS;
        }

WavFile::~WavFile()
        {
        PENTERDES;
        PEXITDES;
        }

int WavFile::open(char* newFileName)
        {
        fileName=newFileName;
        file = fopen(fileName,"r");
        if (!file)
                return -1;
        read_header();
        return 1;
        }


int WavFile::create(char* newFileName)
        {
        audioSize=0;
        headerSize=64;
        fileName=newFileName;
        file = fopen(fileName,"w+");
        if (!file)
                return -1;
        write_header(2,44100,16);
        return 1;
        }


int WavFile::read_header()
        {
        rewind();
        fread(&(audioType), 4, 1 , file);
        fread(&(fileSize), sizeof(long), 1 , file);
        fileSize += 8;
        fread(&(riffType), 4, 1 , file);
        if (strncmp(audioType,"RIFF",4)!=0)
                {
                PERROR("File seems not to be a WAV/RIFF file");
                return -1;
                }
        fread(&(useLess),  8, 1 , file);
        fread(&(compressionCode), sizeof(short), 1, file);
        fread(&(channels), sizeof(short), 1 , file);
        fread(&(rate), sizeof(long), 1 , file);
        fread(&(bytesPerSec), sizeof(long), 1 ,file);
        fread(&(blockSize), sizeof(short), 1 , file);
        fread(&(bitDepth), sizeof(short), 1 , file);
        headerSize=64; /* common wav header size*/
        audioSize = fileSize - headerSize;
        return 1;
        }


int WavFile::write_header(int pChannels, int pRate, int pBitDepth)
        {
        PENTER;
        audioSize=0;
        headerSize=64;

        rewind();
        audioType[0] = 'R';
        audioType[1] = 'I';
        audioType[2] = 'F';
        audioType[3] = 'F';
        fwrite(&(audioType), 4, 1 , file);

        fileSize = audioSize + headerSize;
        fwrite(&(fileSize), sizeof(long), 1 , file);

        riffType[0] = 'W';
        riffType[1] = 'A';
        riffType[2] = 'V';
        riffType[3] = 'E';
        fwrite(&(riffType), 4, 1 , file);

        useLess[0] = 'f';
        useLess[1] = 'm';
        useLess[2] = 't';
        useLess[3] = ' ';
        useLess[4] = 16;
        useLess[5] = 0;
        useLess[6] = 0;
        useLess[7] = 0;
        fwrite(&(useLess),  8, 1 , file);

        compressionCode=1;
        fwrite(&(compressionCode), sizeof(short), 1, file);

        channels=pChannels;
        fwrite(&(channels), sizeof(short), 1 , file);

        rate=pRate;
        fwrite(&(rate), sizeof(long), 1 , file);

        bitDepth=pBitDepth;
        bytesPerSec=rate*channels*bitDepth/8;
        fwrite(&(bytesPerSec), sizeof(long), 1 ,file);
        blockSize=channels*bitDepth/8;
        fwrite(&(blockSize), sizeof(short), 1 , file);

        fwrite(&(bitDepth), sizeof(short), 1 , file);

        useLess[0] = 'd';
        useLess[1] = 'a';
        useLess[2] = 't';
        useLess[3] = 'a';
        useLess[4] = 0x3c;
        useLess[5] = 0x19;
        useLess[6] = 0x59;
        useLess[7] = 0x01;
        fwrite(&(useLess),  8, 1 , file);

        return 1;
        }


void WavFile::rewind()
        {
        std::rewind(file);
        }


void WavFile::skip_header()
        {
        fseek(file, headerSize, SEEK_SET);
        }

FILE* WavFile::get_file()
        {
        return file;
        }

int WavFile::get_bit_depth()
        {
        return bitDepth;
        }


int WavFile::get_header_size()
        {
        return headerSize;
        }

long long WavFile::get_audio_size()
        {
        return audioSize;
        }

int WavFile::get_rate()
        {
        return rate;
        }

int WavFile::get_channels()
        {
        return channels;
        }

void WavFile::show_info()
        {
        /*
        PMESG("File       : " << fileName);
        PMESG("Channels   : " << (channels == 1 ? "Mono" : "Stereo"));
        PMESG("Rate       : " << rate << " Hz");
        PMESG("bitDepth   : " << bitDepth << " bits");
        */
        }

int WavFile::append_samples_from_buffer(char* buffer, int bufferSize)
        {
        size_t size ;
        size = fwrite(buffer, 1 , bufferSize, file);
        int r = 1;
        if ((int) size < bufferSize)
                {
                PERROR("Could not append the whole buffer to file");
                r = -1;
                }
        return r;
        }

int WavFile::fix_audio_size()
        {
        PENTER;
        fseek(file, 0, SEEK_END);
        long long fileSize = ftell(file) - 8;
        fseek(file, 4, SEEK_SET);
        fwrite(&fileSize, 4, 1, file);
        PENTER;
        return 1;
        }

int WavFile::close_file()
        {
        PENTER;
        int r = fclose(file);
        PEXIT;
        return r;
        }
//eof

