/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include "Curve.hh"
#include <stdlib.h>

Curve::Curve(FilterController* pAssocController, QString pType)
        {
        PENTERCONS;
        assocFilterController = pAssocController;
        parentSong = assocFilterController->get_parent_song();
        type = pType;
        PMESG("Creating Curve : %s ", (const char*) pType.latin1());
        r = 0;
        g = 0;
        b = 0;
        PMESG("Adding root node [0:0.00]");
        // Initial node, always added.
        head = new CurveNode( this, 0, 0.0);
        head->prev=(CurveNode*)0;
        head->next=(CurveNode*)0;
        currentNode = head;
        PEXITCONS;
        }


Curve::~Curve()
        {
        PENTERDES;
        CurveNode* n = head;
        while (n)
                {
                CurveNode* n2 = n->next;
                delete n;
                n=n2;
                }
        PEXITDES;
        }

void Curve::set_color(int pr, int pg, int pb )
        {
        r=pr;
        g=pg;
        b=pb;
        }


CurveNode* Curve::add_node(long long p, float v)
        {
        CurveNode* no;
        if (p==0)
                no = head;
        else
                {
                no = new CurveNode( this, p, v);
                add_node(no);
                }
        return no;
        }


void Curve::add_node(CurveNode* node)
        {
        PENTER;
        if (!head)
                {
                head = node;
                node->prev=0;
                node->next=0;
                }
        else
                {
                CurveNode* n1 = head;
                CurveNode* n2 = head;
                while (n1 && (n1->pos < node->pos))
                        {
                        n2=n1;
                        n1=n1->next;
                        }
                if (!n1)
                        {
                        n2->next=node;
                        node->prev=n2;
                        node->next=0;
                        }
                else
                        {
                        node->prev=n2;
                        node->next=n1;
                        n2->next=node;
                        n1->prev=node;
                        }
                }
        currentNode = node;
        PEXIT;
        }




CurveNode* Curve::get_nearest_node(long long pos)
        {
        if (pos<0)
                return head;
        CurveNode* nno = (CurveNode*) 0;
        CurveNode* no = head;
        while (no)
                {
                if (no->next)
                        {
                        if (abs(pos - no->pos) < abs(pos - no->next->pos))
                                {
                                nno=no;
                                break;
                                }
                        }
                else
                        nno=no;
                no=no->next;
                }
        return nno;
        }




void Curve::delete_node(CurveNode* node)
        {
        PENTER;
        PEXIT;
        }


float Curve::get_value_at(long long pos)
// THIS FUNCTION NEEDS HUGE OPTIMIZATION !! It finds the nearest node everytime it needs to get the value. for
// playback this is a waste of CPU. it could "follow" the playback and keep  updating the nearest node for playback cursor during the playback.
        {
        float value=0.0;
        CurveNode* cn = get_nearest_node(pos);
        if (!cn) // MAYBE THIS CHECK IS NOT NECESSARY
                {
                //PDEBUG(" cn==0 !");
                return 0.0;
                }
        if ( (!cn->next) && (pos >= cn->pos)) // after last node, all points use the last node value
                value = cn->value;
        else
                {
                CurveNode* cl;
                CurveNode* cr;
                if ( pos >= cn->pos)
                        {
                        cl = cn;
                        cr = cn->next;
                        }
                else
                        {
                        cl = cn->prev;
                        cr = cn;
                        }
                long long dl = pos - cl->pos;
                long long dd = cr->pos - cl->pos;
                float dv = cr->value - cl->value;
                double qd = (double)dl/(double)dd;
                float dy = dv*(float)qd;
                value = cl->value + dy;
                }
        //PDEBUG("P:"<<pos<<"  V:"<<value);
        return value;
        }

void Curve::show()
        {
        if (!head)
        return;

        int baseY = assocFilterController->get_baseY();
        int half = assocFilterController->get_height()/2;
        QPainter* p = assocFilterController->get_painter();

        CurveNode* n = head;
        int yc = baseY + half;

        int lastx = 0;
        int lasty = yc;

        //if ( n->pos < firstBlock ) continue; FOR LATER
        //if ( n->pos > lastBlock ) break; FOR LATER

        while (n)
                {
                int x = parentSong->block_to_xpos(n->pos);
                int y = yc + (int) (-1*n->value * half / 100 );
                p->setPen(QColor(r, g, b));
                p->drawLine(lastx, lasty, x, y);
                lastx=x;
                lasty=y;
                n=n->next;
                }
        p->setPen(QColor(r, g, b));
        p->drawLine(lastx, lasty, assocFilterController->get_width(), lasty);

        n = head;
        while (n)
                {
                n->draw();
                n=n->next;
                }
        }


void Curve::hide()
        {
        PENTER;
        PEXIT;
        }


void Curve::highlight(int pXpos)
        {
        PENTER;
        PEXIT;
        }


Song* Curve::get_parent_song()
        {
        return parentSong;
        }

CurveNode* Curve::get_current_curve_node()
        {
        return currentNode;
        }

QString Curve::get_type()
        {
        return type;
        }

QString Curve::get_schema()
        {
        QString schema="";
        CurveNode* n = head;
        while (n)
                {
                QString sp; sp.setNum((double)n->pos,'g',10);
                QString sv; sv.setNum(n->value);
                schema=schema+"                 <node pos="+sp+" value="+sv+" />\n";
                n=n->next;
                }
        return schema;
        }
//eof

