/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/

#include "FilterController.hh"

FilterController::FilterController ( Song* pParentSong, MustuxDrawable* pDrawArea, AudioFilter* pFilter ) : MustuxDrawableRegion(pDrawArea)
        {
        PENTERCONS;
        parentSong = pParentSong;
        drawArea=pDrawArea;
        assocFilter = pFilter;
        currentCurveIndex=-1;
        for (int i=0; i<MAX_CURVES; i++) curve[i]=(Curve*) 0;

        // Basic curves : all filters have them
        add_curve("Wet Out"); // this is the master
        add_curve("Pre Gain");
        add_curve("Pos Gain");
        set_current_curve("Wet Out"); // this is the master curve

        blinkCurveTimer = new QTimer(this);
        blinkCurveNodeTimer = new QTimer(this);

        connect( blinkCurveTimer , SIGNAL(timeout()), this, SLOT(blink_curve()));
        connect( blinkCurveNodeTimer , SIGNAL(timeout()), this, SLOT(blink_curve_node()));

        setupFrontend = new QWidget(0);
        setupFrontend->resize(300,200);
        PMESG("this filterController is %p",this);
        PEXITCONS;
        }

FilterController::~FilterController()
        {
        PENTERDES;
        PEXITDES;
        }

void FilterController::setup()
        {
        setupFrontend->show();
        }


void FilterController::node_setup()
        {
        PERROR("Ooops ! I should have not been called ... I am a virtual method!");
        }

int FilterController::get_baseX()
        {
        PERROR("Ooops ! I should have not been called ... I am a virtual method!");
        return -1;
        }

int FilterController::get_baseY()
        {
        PERROR("Ooops ! I should have not been called ... I am a virtual method!");
        return -1;
        }

int FilterController::get_width()
        {
        PERROR("Ooops ! I should have not been called ... I am a virtual method!");
        return -1;
        }

int FilterController::get_height()
        {
        PERROR("Ooops ! I should have not been called ... I am a virtual method!");
        return -1;
        }

void FilterController::set_baseX(int initialValue)
        {
        PERROR("Ooops ! I should have not been called ... I am a virtual method!");
        }

void FilterController::set_baseY(int initialValue)
        {
        PERROR("Ooops ! I should have not been called ... I am a virtual method!");
        }

void FilterController::set_width(int initialValue)
        {
        PERROR("Ooops ! I should have not been called ... I am a virtual method!");
        }

void FilterController::set_height(int initialValue)
        {
        PERROR("Ooops ! I should have not been called ... I am a virtual method!");
        }


Song* FilterController::get_parent_song()
        {
        return parentSong;
        }


QString FilterController::get_type()
        {
        PERROR("Ooops ! I should have not been called ... I am a virtual method!");
        return "";
        }


int FilterController::process(char* data, int size)
        {
        PERROR("Ooops ! I should have not been called ... I am a virtual method!");
        return -1;
        }


void FilterController::draw()
        {
        curve[currentCurveIndex]->show();
        }


void FilterController::update_current_node(int x)
        {
        long long blockPos = parentSong->xpos_to_block(x);
        CurveNode* lastnode = curve[currentCurveIndex]->currentNode;
        if (blockPos>=0)
                {
                CurveNode* cn = curve[currentCurveIndex]->get_nearest_node(blockPos);
                if (cn) curve[currentCurveIndex]->currentNode=cn;
                if ((lastnode != curve[currentCurveIndex]->currentNode) && (lastnode->isHighLighted))
                        lastnode->toggle_highlight();
                }
        }

void FilterController::activate()
        {
        start_blinking();
        }


void FilterController::deactivate()
        {
        stop_blinking();
        }


void FilterController::blink_curve()
        {
        //curve[currentCurveIndex]->...
        }

void FilterController::blink_curve_node()
        {
        curve[currentCurveIndex]->currentNode->toggle_highlight();
        }


void FilterController::start_blinking()
        {
        blinkCurveTimer->start(250);
        blinkCurveNodeTimer->start(250);
        }


void FilterController::stop_blinking()
        {
        blinkCurveTimer->stop();
        blinkCurveNodeTimer->stop();
        if (curve[currentCurveIndex]->currentNode && (curve[currentCurveIndex]->currentNode->isHighLighted))
                curve[currentCurveIndex]->currentNode->toggle_highlight();
        }

Curve* FilterController::get_current_curve()
        {
        return curve[currentCurveIndex];
        }


void FilterController::set_current_curve(QString type)
        {
        for (int i=0; i<MAX_CURVES; i++)
                {
                if (curve[i] && (curve[i]->get_type()==type))
                        {
                        set_current_curve(i);
                        break;
                        }
                }
        }

void FilterController::set_current_curve(int index)
        {
        currentCurveIndex=index;
        }


Curve*  FilterController::get_curve(QString curveType)
        {
        for (int i=0; i<MAX_CURVES; i++)
                {
                if ((curve[i]) && (curveType==curve[i]->get_type()))
                        return curve[i];
                }
        return (Curve*) 0;
        }


int FilterController::get_curve_index(Curve* c)
        {
        for (int i=0; i<MAX_CURVES; i++)
                {
                if ((curve[i]) && (c==curve[i]))
                        return i;
                }
        return -1;
        }


Curve* FilterController::add_curve(QString type)
        {
        int ind=-1;
        for (int i=0; i<MAX_CURVES; i++)
                {
                // cannot add a same filter twice
                if (curve[i] && (curve[i]->get_type()==type))
                        {
                        return (Curve* )0;
                        }
                }

        for (int i=0; i<MAX_CURVES; i++)
                {
                if (!curve[i])
                        {
                        ind=i;
                        break;
                        }
                }
        if (ind>=0)
                {
                curve[ind]=new Curve(this,type);
                set_current_curve(ind);
                return curve[ind];
                }
        return (Curve*) 0;
        }


AudioFilter* FilterController::get_filter()
        {
        return assocFilter;
        }

int FilterController::cleanup()
        {
        assocFilter->cleanup();
        }

QString FilterController::get_schema()
        {
        PENTER;
        QString schema="";
        for (int i=0; i<MAX_CURVES; i++)
                {
                if (curve[i])
                        {
                        schema=schema + "            <curve type=\"" + curve[i]->get_type()+"\">\n";
                        schema=schema + curve[i]->get_schema();
                        schema=schema + "            </curve>\n";
                        }
                }
        PEXIT;
        return schema;
        }

//eof
