#include "NoiseGateController.hh"
#include <mustux.h>

NoiseGateController::NoiseGateController(Song* pParentSong, MustuxDrawable* pDrawArea, AudioFilter* pFilter )
        : FilterController(pParentSong, pDrawArea, pFilter)
        {
        PENTERCONS;
        curveThresholdIndex = get_curve_index(add_curve("Threshold"));
        curveAttackIndex = get_curve_index(add_curve("Attack"));
        curveReleaseIndex = get_curve_index(add_curve("Release"));
        set_current_curve(curveThresholdIndex);
        PEXITCONS;
        }

NoiseGateController::~NoiseGateController()
        {
        // standard routine to delete curves. Copy it to all xxxxController.cc destructors.
        for (int i=0; i<MAX_CURVES; i++)
                if (curve[i]) delete curve[i];
        }

void NoiseGateController::node_setup()
        {
        curve[currentCurveIndex]->get_current_curve_node()->setup();
        }


void NoiseGateController::setup()
        {
        FilterController::setup();

        }


QString NoiseGateController::get_type()
        {
        return "Noise Gate";
        }
int NoiseGateController::get_baseX()
        {
        return baseX;
        }


int NoiseGateController::get_baseY()
        {
        return baseY;
        }

int NoiseGateController::get_width()
        {
        return drawArea->width();
        }

int NoiseGateController::get_height()
        {
        return height;
        }

void NoiseGateController::set_baseX(int initialValue)
        {
        baseX=initialValue;
        }

void NoiseGateController::set_baseY(int initialValue)
        {
        baseY=initialValue;
        }

void NoiseGateController::set_width(int initialValue)
        {
        width=initialValue;
        }

void NoiseGateController::set_height(int initialValue)
        {
        height=initialValue;
        }

void NoiseGateController::draw()
        {
        PENTER;
        FilterController::draw();
        PEXIT;
        }


int NoiseGateController::process(char* data, int size)
        {
        float th = -1.0*(curve[curveThresholdIndex]->get_value_at(parentSong->get_playing_block()) - 100)/2; // 0..100%
        ((NoiseGate*) assocFilter)->set_threshold(th);
        if (assocFilter->load_data(data,size)<0) return -1;
        if (assocFilter->prepare()<0) return -1;
        if (assocFilter->process()<0) return -1;
        return 1;
        }

//eof
